@@ -1,17 +1,21 @@
 from typing import Any, Dict, Optional
 
+from sahi.models.base import DetectionModel
 from sahi.utils.file import import_model_class
 
 MODEL_TYPE_TO_MODEL_CLASS_NAME = {
-    "yolov8": "Yolov8DetectionModel",
+    "ultralytics": "UltralyticsDetectionModel",
+    "rtdetr": "RTDetrDetectionModel",
     "mmdet": "MmdetDetectionModel",
     "yolov5": "Yolov5DetectionModel",
     "detectron2": "Detectron2DetectionModel",
     "huggingface": "HuggingfaceDetectionModel",
     "torchvision": "TorchVisionDetectionModel",
-    "yolov5sparse": "Yolov5SparseDetectionModel",
+    "roboflow": "RoboflowDetectionModel",
 }
 
+ULTRALYTICS_MODEL_NAMES = ["yolov8", "yolov11", "yolo11", "ultralytics"]
+
 
 class AutoDetectionModel:
     @staticmethod
@@ -26,17 +30,19 @@ class AutoDetectionModel:
         category_mapping: Optional[Dict] = None,
         category_remapping: Optional[Dict] = None,
         load_at_init: bool = True,
-        image_size: int = None,
+        image_size: Optional[int] = None,
         **kwargs,
-    ):
+    ) -> DetectionModel:
         """
         Loads a DetectionModel from given path.
 
         Args:
             model_type: str
-                Name of the detection framework (example: "yolov5", "mmdet", "detectron2")
+                Name of the detection framework (example: "ultralytics", "huggingface", "torchvision")
             model_path: str
                 Path of the detection model (ex. 'model.pt')
+            model: Any
+                A pre-initialized model instance, if available
             config_path: str
                 Path of the config file (ex. 'mmdet/configs/cascade_rcnn_r50_fpn_1x.py')
             device: str
@@ -50,15 +56,18 @@ class AutoDetectionModel:
             category_remapping: dict: str to int
                 Remap category ids based on category names, after performing inference e.g. {"car": 3}
             load_at_init: bool
-                If True, automatically loads the model at initalization
+                If True, automatically loads the model at initialization
             image_size: int
                 Inference input size.
+
         Returns:
             Returns an instance of a DetectionModel
+
         Raises:
             ImportError: If given {model_type} framework is not installed
         """
-
+        if model_type in ULTRALYTICS_MODEL_NAMES:
+            model_type = "ultralytics"
         model_class_name = MODEL_TYPE_TO_MODEL_CLASS_NAME[model_type]
         DetectionModel = import_model_class(model_type, model_class_name)
 
