@@ -3,18 +3,19 @@ import fire
 from sahi import __version__ as sahi_version
 from sahi.predict import predict, predict_fiftyone
 from sahi.scripts.coco2fiftyone import main as coco2fiftyone
-from sahi.scripts.coco2yolov5 import main as coco2yolov5
+from sahi.scripts.coco2yolo import main as coco2yolo
 from sahi.scripts.coco_error_analysis import analyse
 from sahi.scripts.coco_evaluation import evaluate
 from sahi.scripts.slice_coco import slice
-from sahi.utils.import_utils import print_enviroment_info
+from sahi.utils.import_utils import print_environment_info
 
 coco_app = {
     "evaluate": evaluate,
     "analyse": analyse,
     "fiftyone": coco2fiftyone,
     "slice": slice,
-    "yolov5": coco2yolov5,
+    "yolo": coco2yolo,
+    "yolov5": coco2yolo,
 }
 
 sahi_app = {
@@ -22,7 +23,7 @@ sahi_app = {
     "predict-fiftyone": predict_fiftyone,
     "coco": coco_app,
     "version": sahi_version,
-    "env": print_enviroment_info,
+    "env": print_environment_info,
 }
 
 
