@@ -1,15 +1,17 @@
-# OBSS SAHI Tool
-# Code written by Fatih C Akyon, 2020.
-
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional
 
 import numpy as np
 
-from sahi.utils.import_utils import is_available
-from sahi.utils.torch import select_device as select_torch_device
+from sahi.annotation import Category
+from sahi.logger import logger
+from sahi.prediction import ObjectPrediction
+from sahi.utils.import_utils import check_requirements
+from sahi.utils.torch_utils import empty_cuda_cache, select_device
 
 
 class DetectionModel:
+    required_packages: List[str] = []
+
     def __init__(
         self,
         model_path: Optional[str] = None,
@@ -21,7 +23,7 @@ class DetectionModel:
         category_mapping: Optional[Dict] = None,
         category_remapping: Optional[Dict] = None,
         load_at_init: bool = True,
-        image_size: int = None,
+        image_size: Optional[int] = None,
     ):
         """
         Init object detection/instance segmentation model.
@@ -30,8 +32,7 @@ class DetectionModel:
                 Path for the instance segmentation model weight
             config_path: str
                 Path for the mmdetection instance segmentation model config file
-            device: str
-                Torch device, "cpu" or "cuda"
+            device: Torch device, "cpu", "mps", "cuda", "cuda:0", "cuda:1", etc.
             mask_threshold: float
                 Value to threshold mask pixels, should be between 0 and 1
             confidence_threshold: float
@@ -41,14 +42,14 @@ class DetectionModel:
             category_remapping: dict: str to int
                 Remap category ids based on category names, after performing inference e.g. {"car": 3}
             load_at_init: bool
-                If True, automatically loads the model at initalization
+                If True, automatically loads the model at initialization
             image_size: int
                 Inference input size.
         """
+
         self.model_path = model_path
         self.config_path = config_path
         self.model = None
-        self.device = device
         self.mask_threshold = mask_threshold
         self.confidence_threshold = confidence_threshold
         self.category_mapping = category_mapping
@@ -56,8 +57,10 @@ class DetectionModel:
         self.image_size = image_size
         self._original_predictions = None
         self._object_prediction_list_per_image = None
+        self.set_device(device)
 
-        self.set_device()
+        # automatically ensure dependencies
+        self.check_dependencies()
 
         # automatically load model if load_at_init is True
         if load_at_init:
@@ -66,11 +69,14 @@ class DetectionModel:
             else:
                 self.load_model()
 
-    def check_dependencies(self) -> None:
+    def check_dependencies(self, packages: Optional[List[str]] = None) -> None:
         """
-        This function can be implemented to ensure model dependencies are installed.
+        Ensures required dependencies are installed. If 'packages' is None, uses self.required_packages.
+        Subclasses may still call with a custom list for dynamic needs.
         """
-        pass
+        pkgs = packages if packages is not None else getattr(self, "required_packages", [])
+        if pkgs:
+            check_requirements(pkgs)
 
     def load_model(self):
         """
@@ -89,24 +95,21 @@ class DetectionModel:
         """
         raise NotImplementedError()
 
-    def set_device(self):
+    def set_device(self, device: Optional[str] = None):
+        """Sets the device pytorch should use for the model
+
+        Args:
+            device: Torch device, "cpu", "mps", "cuda", "cuda:0", "cuda:1", etc.
         """
-        Sets the device for the model.
-        """
-        if is_available("torch"):
-            self.device = select_torch_device(self.device)
-        else:
-            raise NotImplementedError()
+
+        self.device = select_device(device)
 
     def unload_model(self):
         """
         Unloads the model from CPU/GPU.
         """
         self.model = None
-        if is_available("torch"):
-            from sahi.utils.torch import empty_cuda_cache
-
-            empty_cuda_cache()
+        empty_cuda_cache()
 
     def perform_inference(self, image: np.ndarray):
         """
@@ -145,16 +148,21 @@ class DetectionModel:
         if self.category_remapping is None:
             raise ValueError("self.category_remapping cannot be None")
         # remap categories
-        for object_prediction_list in self._object_prediction_list_per_image:
+        if not isinstance(self._object_prediction_list_per_image, list):
+            logger.error(
+                f"Unknown type for self._object_prediction_list_per_image: {type(self._object_prediction_list_per_image)}"
+            )
+            return
+        for object_prediction_list in self._object_prediction_list_per_image:  # type: ignore
             for object_prediction in object_prediction_list:
                 old_category_id_str = str(object_prediction.category.id)
                 new_category_id_int = self.category_remapping[old_category_id_str]
-                object_prediction.category.id = new_category_id_int
+                object_prediction.category = Category(id=new_category_id_int, name=object_prediction.category.name)
 
     def convert_original_predictions(
         self,
-        shift_amount: Optional[List[int]] = [0, 0],
-        full_shape: Optional[List[int]] = None,
+        shift_amount: Optional[List[List[int]]] = [[0, 0]],
+        full_shape: Optional[List[List[int]]] = None,
     ):
         """
         Converts original predictions of the detection model to a list of
@@ -173,12 +181,16 @@ class DetectionModel:
             self._apply_category_remapping()
 
     @property
-    def object_prediction_list(self):
+    def object_prediction_list(self) -> List[List[ObjectPrediction]]:
+        if self._object_prediction_list_per_image is None:
+            return []
+        if len(self._object_prediction_list_per_image) == 0:
+            return []
         return self._object_prediction_list_per_image[0]
 
     @property
-    def object_prediction_list_per_image(self):
-        return self._object_prediction_list_per_image
+    def object_prediction_list_per_image(self) -> List[List[ObjectPrediction]]:
+        return self._object_prediction_list_per_image or []
 
     @property
     def original_predictions(self):
