@@ -1,18 +1,13 @@
-# OBSS SAHI Tool
-# Code written by Fatih C Akyon, 2020.
-
-import logging
-from typing import Any, Dict, List, Optional
+from typing import Any, List, Optional
 
 import numpy as np
 
+from sahi.logger import logger
 from sahi.models.base import DetectionModel
 from sahi.prediction import ObjectPrediction
 from sahi.utils.compatibility import fix_full_shape_list, fix_shift_amount_list
 from sahi.utils.import_utils import check_package_minimum_version, check_requirements
 
-logger = logging.getLogger(__name__)
-
 
 class Yolov5DetectionModel(DetectionModel):
     def check_dependencies(self) -> None:
@@ -157,7 +152,7 @@ class Yolov5DetectionModel(DetectionModel):
                     bbox=bbox,
                     category_id=category_id,
                     score=score,
-                    bool_mask=None,
+                    segmentation=None,
                     category_name=category_name,
                     shift_amount=shift_amount,
                     full_shape=full_shape,
