@@ -1,16 +1,16 @@
-# OBSS SAHI Tool
-# Code written by Fatih C Akyon, 2020.
-
-import logging
 import os
 import time
-from typing import List, Optional
+from typing import Generator, List, Optional, Union
 
+from PIL import Image
+
+from sahi.logger import logger
 from sahi.utils.import_utils import is_available
 
+# TODO: This does nothing for this module. The issue named here does not exist
 # https://github.com/obss/sahi/issues/526
 if is_available("torch"):
-    import torch
+    import torch  # noqa: F401
 
 from functools import cmp_to_key
 
@@ -51,7 +51,13 @@ POSTPROCESS_NAME_TO_CLASS = {
 LOW_MODEL_CONFIDENCE = 0.1
 
 
-logger = logging.getLogger(__name__)
+def filter_predictions(object_prediction_list, exclude_classes_by_name, exclude_classes_by_id):
+    return [
+        obj_pred
+        for obj_pred in object_prediction_list
+        if obj_pred.category.name not in (exclude_classes_by_name or [])
+        and obj_pred.category.id not in (exclude_classes_by_id or [])
+    ]
 
 
 def get_prediction(
@@ -61,6 +67,8 @@ def get_prediction(
     full_shape=None,
     postprocess: Optional[PostprocessPredictions] = None,
     verbose: int = 0,
+    exclude_classes_by_name: Optional[List[str]] = None,
+    exclude_classes_by_id: Optional[List[int]] = None,
 ) -> PredictionResult:
     """
     Function for performing prediction for given image using given detection_model.
@@ -78,7 +86,12 @@ def get_prediction(
         verbose: int
             0: no print (default)
             1: print prediction duration
-
+        exclude_classes_by_name: Optional[List[str]]
+            None: if no classes are excluded
+            List[str]: set of classes to exclude using its/their class label name/s
+        exclude_classes_by_id: Optional[List[int]]
+            None: if no classes are excluded
+            List[int]: set of classes to exclude using one or more IDs
     Returns:
         A dict with fields:
             object_prediction_list: a list of ObjectPrediction
@@ -94,6 +107,9 @@ def get_prediction(
     time_end = time.time() - time_start
     durations_in_seconds["prediction"] = time_end
 
+    if full_shape is None:
+        full_shape = [image_as_pil.height, image_as_pil.width]
+
     # process prediction
     time_start = time.time()
     # works only with 1 batch
@@ -102,6 +118,7 @@ def get_prediction(
         full_shape=full_shape,
     )
     object_prediction_list: List[ObjectPrediction] = detection_model.object_prediction_list
+    object_prediction_list = filter_predictions(object_prediction_list, exclude_classes_by_name, exclude_classes_by_id)
 
     # postprocess matching predictions
     if postprocess is not None:
@@ -125,8 +142,8 @@ def get_prediction(
 def get_sliced_prediction(
     image,
     detection_model=None,
-    slice_height: int = None,
-    slice_width: int = None,
+    slice_height: Optional[int] = None,
+    slice_width: Optional[int] = None,
     overlap_height_ratio: float = 0.2,
     overlap_width_ratio: float = 0.2,
     perform_standard_pred: bool = True,
@@ -135,8 +152,12 @@ def get_sliced_prediction(
     postprocess_match_threshold: float = 0.5,
     postprocess_class_agnostic: bool = False,
     verbose: int = 1,
-    merge_buffer_length: int = None,
+    merge_buffer_length: Optional[int] = None,
     auto_slice_resolution: bool = True,
+    slice_export_prefix: Optional[str] = None,
+    slice_dir: Optional[str] = None,
+    exclude_classes_by_name: Optional[List[str]] = None,
+    exclude_classes_by_id: Optional[List[int]] = None,
 ) -> PredictionResult:
     """
     Function for slice image + get predicion for each slice + combine predictions in full image.
@@ -162,7 +183,7 @@ def get_sliced_prediction(
             detection accuracy. Default: True.
         postprocess_type: str
             Type of the postprocess to be used after sliced inference while merging/eliminating predictions.
-            Options are 'NMM', 'GRREDYNMM' or 'NMS'. Default is 'GRREDYNMM'.
+            Options are 'NMM', 'GREEDYNMM' or 'NMS'. Default is 'GREEDYNMM'.
         postprocess_match_metric: str
             Metric to be used during object prediction matching after sliced prediction.
             'IOU' for intersection over union, 'IOS' for intersection over smaller area.
@@ -182,7 +203,16 @@ def get_sliced_prediction(
         auto_slice_resolution: bool
             if slice parameters (slice_height, slice_width) are not given,
             it enables automatically calculate these params from image resolution and orientation.
-
+        slice_export_prefix: str
+            Prefix for the exported slices. Defaults to None.
+        slice_dir: str
+            Directory to save the slices. Defaults to None.
+        exclude_classes_by_name: Optional[List[str]]
+            None: if no classes are excluded
+            List[str]: set of classes to exclude using its/their class label name/s
+        exclude_classes_by_id: Optional[List[int]]
+            None: if no classes are excluded
+            List[int]: set of classes to exclude using one or more IDs
     Returns:
         A Dict with fields:
             object_prediction_list: a list of sahi.prediction.ObjectPrediction
@@ -194,29 +224,33 @@ def get_sliced_prediction(
 
     # currently only 1 batch supported
     num_batch = 1
-
     # create slices from full image
     time_start = time.time()
     slice_image_result = slice_image(
         image=image,
+        output_file_name=slice_export_prefix,
+        output_dir=slice_dir,
         slice_height=slice_height,
         slice_width=slice_width,
         overlap_height_ratio=overlap_height_ratio,
         overlap_width_ratio=overlap_width_ratio,
         auto_slice_resolution=auto_slice_resolution,
     )
+    from sahi.models.ultralytics import UltralyticsDetectionModel
+
     num_slices = len(slice_image_result)
     time_end = time.time() - time_start
     durations_in_seconds["slice"] = time_end
 
+    if isinstance(detection_model, UltralyticsDetectionModel) and detection_model.is_obb:
+        # Only NMS is supported for OBB model outputs
+        postprocess_type = "NMS"
+
     # init match postprocess instance
     if postprocess_type not in POSTPROCESS_NAME_TO_CLASS.keys():
         raise ValueError(
             f"postprocess_type should be one of {list(POSTPROCESS_NAME_TO_CLASS.keys())} but given as {postprocess_type}"
         )
-    elif postprocess_type == "UNIONMERGE":
-        # deprecated in v0.9.3
-        raise ValueError("'UNIONMERGE' postprocess_type is deprecated, use 'GREEDYNMM' instead.")
     postprocess_constructor = POSTPROCESS_NAME_TO_CLASS[postprocess_type]
     postprocess = postprocess_constructor(
         match_threshold=postprocess_match_threshold,
@@ -224,10 +258,13 @@ def get_sliced_prediction(
         class_agnostic=postprocess_class_agnostic,
     )
 
+    postprocess_time = 0
+    time_start = time.time()
+
     # create prediction input
     num_group = int(num_slices / num_batch)
     if verbose == 1 or verbose == 2:
-        tqdm.write(f"Performing prediction on {num_slices} number of slices.")
+        tqdm.write(f"Performing prediction on {num_slices} slices.")
     object_prediction_list = []
     # perform sliced prediction
     for group_ind in range(num_group):
@@ -246,6 +283,8 @@ def get_sliced_prediction(
                 slice_image_result.original_image_height,
                 slice_image_result.original_image_width,
             ],
+            exclude_classes_by_name=exclude_classes_by_name,
+            exclude_classes_by_id=exclude_classes_by_id,
         )
         # convert sliced predictions to full predictions
         for object_prediction in prediction_result.object_prediction_list:
@@ -254,7 +293,9 @@ def get_sliced_prediction(
 
         # merge matching predictions during sliced prediction
         if merge_buffer_length is not None and len(object_prediction_list) > merge_buffer_length:
+            postprocess_time_start = time.time()
             object_prediction_list = postprocess(object_prediction_list)
+            postprocess_time += time.time() - postprocess_time_start
 
     # perform standard prediction
     if num_slices > 1 and perform_standard_pred:
@@ -262,17 +303,25 @@ def get_sliced_prediction(
             image=image,
             detection_model=detection_model,
             shift_amount=[0, 0],
-            full_shape=None,
+            full_shape=[
+                slice_image_result.original_image_height,
+                slice_image_result.original_image_width,
+            ],
             postprocess=None,
+            exclude_classes_by_name=exclude_classes_by_name,
+            exclude_classes_by_id=exclude_classes_by_id,
         )
         object_prediction_list.extend(prediction_result.object_prediction_list)
 
     # merge matching predictions
     if len(object_prediction_list) > 1:
+        postprocess_time_start = time.time()
         object_prediction_list = postprocess(object_prediction_list)
+        postprocess_time += time.time() - postprocess_time_start
 
     time_end = time.time() - time_start
-    durations_in_seconds["prediction"] = time_end
+    durations_in_seconds["prediction"] = time_end - postprocess_time
+    durations_in_seconds["postprocess"] = postprocess_time
 
     if verbose == 2:
         print(
@@ -285,6 +334,11 @@ def get_sliced_prediction(
             durations_in_seconds["prediction"],
             "seconds.",
         )
+        print(
+            "Postprocessing performed in",
+            durations_in_seconds["postprocess"],
+            "seconds.",
+        )
 
     return PredictionResult(
         image=image, object_prediction_list=object_prediction_list, durations_in_seconds=durations_in_seconds
@@ -329,18 +383,18 @@ def agg_prediction(result: PredictionResult, thresh):
 
 
 def predict(
-    detection_model: DetectionModel = None,
-    model_type: str = "mmdet",
-    model_path: str = None,
-    model_config_path: str = None,
+    detection_model: Optional[DetectionModel] = None,
+    model_type: str = "ultralytics",
+    model_path: Optional[str] = None,
+    model_config_path: Optional[str] = None,
     model_confidence_threshold: float = 0.25,
-    model_device: str = None,
-    model_category_mapping: dict = None,
-    model_category_remapping: dict = None,
-    source: str = None,
+    model_device: Optional[str] = None,
+    model_category_mapping: Optional[dict] = None,
+    model_category_remapping: Optional[dict] = None,
+    source: Optional[str] = None,
     no_standard_prediction: bool = False,
     no_sliced_prediction: bool = False,
-    image_size: int = None,
+    image_size: Optional[int] = None,
     slice_height: int = 512,
     slice_width: int = 512,
     overlap_height_ratio: float = 0.2,
@@ -354,18 +408,21 @@ def predict(
     frame_skip_interval: int = 0,
     export_pickle: bool = False,
     export_crop: bool = False,
-    dataset_json_path: bool = None,
+    dataset_json_path: Optional[str] = None,
     project: str = "runs/predict",
     name: str = "exp",
-    visual_bbox_thickness: int = None,
-    visual_text_size: float = None,
-    visual_text_thickness: int = None,
+    visual_bbox_thickness: Optional[int] = None,
+    visual_text_size: Optional[float] = None,
+    visual_text_thickness: Optional[int] = None,
     visual_hide_labels: bool = False,
     visual_hide_conf: bool = False,
     visual_export_format: str = "png",
     verbose: int = 1,
     return_dict: bool = False,
     force_postprocess_type: bool = False,
+    exclude_classes_by_name: Optional[List[str]] = None,
+    exclude_classes_by_id: Optional[List[int]] = None,
+    **kwargs,
 ):
     """
     Performs prediction for all present images in given folder.
@@ -411,7 +468,7 @@ def predict(
             Default to ``0.2``.
         postprocess_type: str
             Type of the postprocess to be used after sliced inference while merging/eliminating predictions.
-            Options are 'NMM', 'GRREDYNMM' or 'NMS'. Default is 'GRREDYNMM'.
+            Options are 'NMM', 'GREEDYNMM', 'LSNMS' or 'NMS'. Default is 'GREEDYNMM'.
         postprocess_match_metric: str
             Metric to be used during object prediction matching after sliced prediction.
             'IOU' for intersection over union, 'IOS' for intersection over smaller area.
@@ -451,6 +508,12 @@ def predict(
             If True, returns a dict with 'export_dir' field.
         force_postprocess_type: bool
             If True, auto postprocess check will e disabled
+        exclude_classes_by_name: Optional[List[str]]
+            None: if no classes are excluded
+            List[str]: set of classes to exclude using its/their class label name/s
+        exclude_classes_by_id: Optional[List[int]]
+            None: if no classes are excluded
+            List[int]: set of classes to exclude using one or more IDs
     """
     # assert prediction type
     if no_standard_prediction and no_sliced_prediction:
@@ -480,24 +543,24 @@ def predict(
     # TODO: rewrite this as iterator class as in https://github.com/ultralytics/yolov5/blob/d059d1da03aee9a3c0059895aa4c7c14b7f25a9e/utils/datasets.py#L178
     source_is_video = False
     num_frames = None
-    if dataset_json_path:
+    image_iterator: Union[list[str], Generator[Image.Image, None, None]]
+    if dataset_json_path and source:
         coco: Coco = Coco.from_coco_dict_or_path(dataset_json_path)
         image_iterator = [str(Path(source) / Path(coco_image.file_name)) for coco_image in coco.images]
         coco_json = []
-    elif os.path.isdir(source):
-        image_iterator = list_files(
-            directory=source,
-            contains=IMAGE_EXTENSIONS,
-            verbose=verbose,
-        )
-    elif Path(source).suffix in VIDEO_EXTENSIONS:
+    elif source and os.path.isdir(source):
+        image_iterator = list_files(directory=source, contains=IMAGE_EXTENSIONS, verbose=verbose)
+    elif source and Path(source).suffix in VIDEO_EXTENSIONS:
         source_is_video = True
         read_video_frame, output_video_writer, video_file_name, num_frames = get_video_reader(
-            source, save_dir, frame_skip_interval, not novisual, view_video
+            source, str(save_dir), frame_skip_interval, not novisual, view_video
         )
         image_iterator = read_video_frame
-    else:
+    elif source:
         image_iterator = [source]
+    else:
+        logger.error("No valid input given to predict function")
+        return
 
     # init model instance
     time_start = time.time()
@@ -512,6 +575,7 @@ def predict(
             category_remapping=model_category_remapping,
             load_at_init=False,
             image_size=image_size,
+            **kwargs,
         )
         detection_model.load_model()
     time_end = time.time() - time_start
@@ -525,11 +589,14 @@ def predict(
     for ind, image_path in enumerate(
         tqdm(image_iterator, f"Performing inference on {input_type_str}", total=num_frames)
     ):
-        # get filename
-        if source_is_video:
+        # Source is an image: Iterating over Image objects
+        if source and source_is_video:
             video_name = Path(source).stem
             relative_filepath = video_name + "_frame_" + str(ind)
-        elif os.path.isdir(source):  # preserve source folder structure in export
+        elif isinstance(image_path, Image.Image):
+            raise RuntimeError("Source is not a video, but image is still an Image object ")
+        # preserve source folder structure in export
+        elif source and os.path.isdir(source):
             relative_filepath = str(Path(image_path)).split(str(Path(source)))[-1]
             relative_filepath = relative_filepath[1:] if relative_filepath[0] == os.sep else relative_filepath
         else:  # no process if source is single file
@@ -556,9 +623,12 @@ def predict(
                 postprocess_match_threshold=postprocess_match_threshold,
                 postprocess_class_agnostic=postprocess_class_agnostic,
                 verbose=1 if verbose else 0,
+                exclude_classes_by_name=exclude_classes_by_name,
+                exclude_classes_by_id=exclude_classes_by_id,
             )
             object_prediction_list = prediction_result.object_prediction_list
-            durations_in_seconds["slice"] += prediction_result.durations_in_seconds["slice"]
+            if prediction_result.durations_in_seconds:
+                durations_in_seconds["slice"] += prediction_result.durations_in_seconds["slice"]
         else:
             # get standard prediction
             prediction_result = get_prediction(
@@ -568,6 +638,8 @@ def predict(
                 full_shape=None,
                 postprocess=None,
                 verbose=0,
+                exclude_classes_by_name=exclude_classes_by_name,
+                exclude_classes_by_id=exclude_classes_by_id,
             )
             object_prediction_list = prediction_result.object_prediction_list
 
@@ -664,7 +736,9 @@ def predict(
                 export_format=visual_export_format,
             )
             if not novisual and source_is_video:  # export video
-                output_video_writer.write(result["image"])
+                if output_video_writer is None:
+                    raise RuntimeError("Output video writer could not be created")
+                output_video_writer.write(cv2.cvtColor(result["image"], cv2.COLOR_RGB2BGR))
 
         # render video inference
         if view_video:
@@ -712,17 +786,17 @@ def predict(
 
 def predict_fiftyone(
     model_type: str = "mmdet",
-    model_path: str = None,
-    model_config_path: str = None,
+    model_path: Optional[str] = None,
+    model_config_path: Optional[str] = None,
     model_confidence_threshold: float = 0.25,
-    model_device: str = None,
-    model_category_mapping: dict = None,
-    model_category_remapping: dict = None,
-    dataset_json_path: str = None,
-    image_dir: str = None,
+    model_device: Optional[str] = None,
+    model_category_mapping: Optional[dict] = None,
+    model_category_remapping: Optional[dict] = None,
+    dataset_json_path: str = "",
+    image_dir: str = "",
     no_standard_prediction: bool = False,
     no_sliced_prediction: bool = False,
-    image_size: int = None,
+    image_size: Optional[int] = None,
     slice_height: int = 256,
     slice_width: int = 256,
     overlap_height_ratio: float = 0.2,
@@ -732,6 +806,8 @@ def predict_fiftyone(
     postprocess_match_threshold: float = 0.5,
     postprocess_class_agnostic: bool = False,
     verbose: int = 1,
+    exclude_classes_by_name: Optional[List[str]] = None,
+    exclude_classes_by_id: Optional[List[int]] = None,
 ):
     """
     Performs prediction for all present images in given folder.
@@ -775,7 +851,7 @@ def predict_fiftyone(
             Default to ``0.2``.
         postprocess_type: str
             Type of the postprocess to be used after sliced inference while merging/eliminating predictions.
-            Options are 'NMM', 'GRREDYNMM' or 'NMS'. Default is 'GRREDYNMM'.
+            Options are 'NMM', 'GREEDYNMM' or 'NMS'. Default is 'GREEDYNMM'.
         postprocess_match_metric: str
             Metric to be used during object prediction matching after sliced prediction.
             'IOU' for intersection over union, 'IOS' for intersection over smaller area.
@@ -790,6 +866,12 @@ def predict_fiftyone(
         verbose: int
             0: no print
             1: print slice/prediction durations, number of slices, model loading/file exporting durations
+        exclude_classes_by_name: Optional[List[str]]
+            None: if no classes are excluded
+            List[str]: set of classes to exclude using its/their class label name/s
+        exclude_classes_by_id: Optional[List[int]]
+            None: if no classes are excluded
+            List[int]: set of classes to exclude using one or more IDs
     """
     check_requirements(["fiftyone"])
 
@@ -842,6 +924,8 @@ def predict_fiftyone(
                     postprocess_match_metric=postprocess_match_metric,
                     postprocess_class_agnostic=postprocess_class_agnostic,
                     verbose=verbose,
+                    exclude_classes_by_name=exclude_classes_by_name,
+                    exclude_classes_by_id=exclude_classes_by_id,
                 )
                 durations_in_seconds["slice"] += prediction_result.durations_in_seconds["slice"]
             else:
@@ -853,6 +937,8 @@ def predict_fiftyone(
                     full_shape=None,
                     postprocess=None,
                     verbose=0,
+                    exclude_classes_by_name=exclude_classes_by_name,
+                    exclude_classes_by_id=exclude_classes_by_id,
                 )
                 durations_in_seconds["prediction"] += prediction_result.durations_in_seconds["prediction"]
 
@@ -879,7 +965,7 @@ def predict_fiftyone(
         )
 
     # visualize results
-    session = fo.launch_app()
+    session = fo.launch_app()  # pyright: ignore[reportArgumentType]
     session.dataset = dataset
     # Evaluate the predictions
     results = dataset.evaluate_detections(
