@@ -1,12 +1,7 @@
-# OBSS SAHI Tool
-# Code written by Fatih C Akyon, 2020.
-
 import concurrent.futures
-import logging
 import os
-import time
 from pathlib import Path
-from typing import Dict, List, Optional, Sequence, Union
+from typing import Dict, List, Literal, Optional, Sequence, Tuple, Union
 
 import numpy as np
 from PIL import Image
@@ -14,46 +9,43 @@ from shapely.errors import TopologicalError
 from tqdm import tqdm
 
 from sahi.annotation import BoundingBox, Mask
+from sahi.logger import logger
 from sahi.utils.coco import Coco, CocoAnnotation, CocoImage, create_coco_dict
-from sahi.utils.cv import read_image_as_pil
+from sahi.utils.cv import IMAGE_EXTENSIONS_LOSSLESS, IMAGE_EXTENSIONS_LOSSY, read_image_as_pil
 from sahi.utils.file import load_json, save_json
 
-logger = logging.getLogger(__name__)
-logging.basicConfig(
-    format="%(asctime)s - %(levelname)s - %(name)s -   %(message)s",
-    datefmt="%m/%d/%Y %H:%M:%S",
-    level=os.environ.get("LOGLEVEL", "INFO").upper(),
-)
-
 MAX_WORKERS = 20
 
 
 def get_slice_bboxes(
     image_height: int,
     image_width: int,
-    slice_height: int = None,
-    slice_width: int = None,
-    auto_slice_resolution: bool = True,
-    overlap_height_ratio: float = 0.2,
-    overlap_width_ratio: float = 0.2,
+    slice_height: Optional[int] = None,
+    slice_width: Optional[int] = None,
+    auto_slice_resolution: Optional[bool] = True,
+    overlap_height_ratio: Optional[float] = 0.2,
+    overlap_width_ratio: Optional[float] = 0.2,
 ) -> List[List[int]]:
-    """Slices `image_pil` in crops.
-    Corner values of each slice will be generated using the `slice_height`,
-    `slice_width`, `overlap_height_ratio` and `overlap_width_ratio` arguments.
+    """Generate bounding boxes for slicing an image into crops.
+
+    The function calculates the coordinates for each slice based on the provided
+    image dimensions, slice size, and overlap ratios. If slice size is not provided
+    and auto_slice_resolution is True, the function will automatically determine
+    appropriate slice parameters.
 
     Args:
         image_height (int): Height of the original image.
         image_width (int): Width of the original image.
-        slice_height (int): Height of each slice. Default 512.
-        slice_width (int): Width of each slice. Default 512.
-        overlap_height_ratio(float): Fractional overlap in height of each
+        slice_height (int, optional): Height of each slice. Default None.
+        slice_width (int, optional): Width of each slice. Default None.
+        overlap_height_ratio (float, optional): Fractional overlap in height of each
             slice (e.g. an overlap of 0.2 for a slice of size 100 yields an
             overlap of 20 pixels). Default 0.2.
-        overlap_width_ratio(float): Fractional overlap in width of each
+        overlap_width_ratio(float, optional): Fractional overlap in width of each
             slice (e.g. an overlap of 0.2 for a slice of size 100 yields an
             overlap of 20 pixels). Default 0.2.
-        auto_slice_resolution (bool): if not set slice parameters such as slice_height and slice_width,
-            it enables automatically calculate these params from image resolution and orientation.
+        auto_slice_resolution (bool, optional): if not set slice parameters such as slice_height and slice_width,
+            it enables automatically calculate these parameters from image resolution and orientation.
 
     Returns:
         List[List[int]]: List of 4 corner coordinates for each N slices.
@@ -120,7 +112,9 @@ def annotation_inside_slice(annotation: Dict, slice_bbox: List[int]) -> bool:
     return True
 
 
-def process_coco_annotations(coco_annotation_list: List[CocoAnnotation], slice_bbox: List[int], min_area_ratio) -> bool:
+def process_coco_annotations(
+    coco_annotation_list: List[CocoAnnotation], slice_bbox: List[int], min_area_ratio
+) -> List[CocoAnnotation]:
     """Slices and filters given list of CocoAnnotation objects with given
     'slice_bbox' and 'min_area_ratio'.
 
@@ -161,19 +155,19 @@ class SlicedImage:
 
 
 class SliceImageResult:
-    def __init__(self, original_image_size=None, image_dir: str = None):
+    def __init__(self, original_image_size: List[int], image_dir: Optional[str] = None):
         """
-        sliced_image_list: list of SlicedImage
         image_dir: str
             Directory of the sliced image exports.
         original_image_size: list of int
             Size of the unsliced original image in [height, width]
         """
-        self._sliced_image_list: List[SlicedImage] = []
         self.original_image_height = original_image_size[0]
         self.original_image_width = original_image_size[1]
         self.image_dir = image_dir
 
+        self._sliced_image_list: List[SlicedImage] = []
+
     def add_sliced_image(self, sliced_image: SlicedImage):
         if not isinstance(sliced_image, SlicedImage):
             raise TypeError("sliced_image must be a SlicedImage instance")
@@ -261,43 +255,45 @@ class SliceImageResult:
 
 def slice_image(
     image: Union[str, Image.Image],
-    coco_annotation_list: Optional[CocoAnnotation] = None,
+    coco_annotation_list: Optional[List[CocoAnnotation]] = None,
     output_file_name: Optional[str] = None,
     output_dir: Optional[str] = None,
-    slice_height: int = None,
-    slice_width: int = None,
-    overlap_height_ratio: float = None,
-    overlap_width_ratio: float = None,
-    auto_slice_resolution: bool = True,
-    min_area_ratio: float = 0.1,
+    slice_height: Optional[int] = None,
+    slice_width: Optional[int] = None,
+    overlap_height_ratio: Optional[float] = 0.2,
+    overlap_width_ratio: Optional[float] = 0.2,
+    auto_slice_resolution: Optional[bool] = True,
+    min_area_ratio: Optional[float] = 0.1,
     out_ext: Optional[str] = None,
-    verbose: bool = False,
+    verbose: Optional[bool] = False,
+    exif_fix: bool = True,
 ) -> SliceImageResult:
-    """Slice a large image into smaller windows. If output_file_name is given export
+    """Slice a large image into smaller windows. If output_file_name and output_dir is given, export
     sliced images.
 
     Args:
         image (str or PIL.Image): File path of image or Pillow Image to be sliced.
-        coco_annotation_list (CocoAnnotation): List of CocoAnnotation objects.
+        coco_annotation_list (List[CocoAnnotation], optional): List of CocoAnnotation objects.
         output_file_name (str, optional): Root name of output files (coordinates will
             be appended to this)
         output_dir (str, optional): Output directory
-        slice_height (int): Height of each slice. Default 512.
-        slice_width (int): Width of each slice. Default 512.
-        overlap_height_ratio (float): Fractional overlap in height of each
+        slice_height (int, optional): Height of each slice. Default None.
+        slice_width (int, optional): Width of each slice. Default None.
+        overlap_height_ratio (float, optional): Fractional overlap in height of each
             slice (e.g. an overlap of 0.2 for a slice of size 100 yields an
             overlap of 20 pixels). Default 0.2.
-        overlap_width_ratio (float): Fractional overlap in width of each
+        overlap_width_ratio (float, optional): Fractional overlap in width of each
             slice (e.g. an overlap of 0.2 for a slice of size 100 yields an
             overlap of 20 pixels). Default 0.2.
-        auto_slice_resolution (bool): if not set slice parameters such as slice_height and slice_width,
+        auto_slice_resolution (bool, optional): if not set slice parameters such as slice_height and slice_width,
             it enables automatically calculate these params from image resolution and orientation.
-        min_area_ratio (float): If the cropped annotation area to original annotation
+        min_area_ratio (float, optional): If the cropped annotation area to original annotation
             ratio is smaller than this value, the annotation is filtered out. Default 0.1.
         out_ext (str, optional): Extension of saved images. Default is the
-            original suffix.
+            original suffix for lossless image formats and png for lossy formats ('.jpg','.jpeg').
         verbose (bool, optional): Switch to print relevant values to screen.
             Default 'False'.
+        exif_fix (bool): Whether to apply an EXIF fix to the image.
 
     Returns:
         sliced_image_result: SliceImageResult:
@@ -306,15 +302,13 @@ def slice_image(
                                     Directory of the sliced image exports.
                                 original_image_size: list of int
                                     Size of the unsliced original image in [height, width]
-        num_total_invalid_segmentation: int
-            Number of invalid segmentation annotations.
     """
 
     # define verboseprint
     verboselog = logger.info if verbose else lambda *a, **k: None
 
     def _export_single_slice(image: np.ndarray, output_dir: str, slice_file_name: str):
-        image_pil = read_image_as_pil(image)
+        image_pil = read_image_as_pil(image, exif_fix=exif_fix)
         slice_file_path = str(Path(output_dir) / slice_file_name)
         # export sliced image
         image_pil.save(slice_file_path)
@@ -326,7 +320,7 @@ def slice_image(
         Path(output_dir).mkdir(parents=True, exist_ok=True)
 
     # read image
-    image_pil = read_image_as_pil(image)
+    image_pil = read_image_as_pil(image, exif_fix=exif_fix)
     verboselog("image.shape: " + str(image_pil.size))
 
     image_width, image_height = image_pil.size
@@ -342,7 +336,6 @@ def slice_image(
         overlap_width_ratio=overlap_width_ratio,
     )
 
-    t0 = time.time()
     n_ims = 0
 
     # init images and annotations lists
@@ -360,19 +353,18 @@ def slice_image(
         bry = slice_bbox[3]
         image_pil_slice = image_pil_arr[tly:bry, tlx:brx]
 
-        # process annotations if coco_annotations is given
-        if coco_annotation_list is not None:
-            sliced_coco_annotation_list = process_coco_annotations(coco_annotation_list, slice_bbox, min_area_ratio)
-
         # set image file suffixes
         slice_suffixes = "_".join(map(str, slice_bbox))
         if out_ext:
             suffix = out_ext
-        else:
-            try:
+        elif hasattr(image_pil, "filename"):
+            suffix = Path(getattr(image_pil, "filename")).suffix
+            if suffix in IMAGE_EXTENSIONS_LOSSY:
+                suffix = ".png"
+            elif suffix in IMAGE_EXTENSIONS_LOSSLESS:
                 suffix = Path(image_pil.filename).suffix
-            except AttributeError:
-                suffix = ".jpg"
+        else:
+            suffix = ".png"
 
         # set image file name and path
         slice_file_name = f"{output_file_name}_{slice_suffixes}{suffix}"
@@ -383,9 +375,9 @@ def slice_image(
         coco_image = CocoImage(file_name=slice_file_name, height=slice_height, width=slice_width)
 
         # append coco annotations (if present) to coco image
-        if coco_annotation_list:
-            for coco_annotation in sliced_coco_annotation_list:
-                coco_image.add_annotation(coco_annotation)
+        if coco_annotation_list is not None:
+            for sliced_coco_annotation in process_coco_annotations(coco_annotation_list, slice_bbox, min_area_ratio):
+                coco_image.add_annotation(sliced_coco_annotation)
 
         # create sliced image and append to sliced_image_result
         sliced_image = SlicedImage(
@@ -415,32 +407,33 @@ def slice_coco(
     image_dir: str,
     output_coco_annotation_file_name: str,
     output_dir: Optional[str] = None,
-    ignore_negative_samples: bool = False,
-    slice_height: int = 512,
-    slice_width: int = 512,
-    overlap_height_ratio: float = 0.2,
-    overlap_width_ratio: float = 0.2,
-    min_area_ratio: float = 0.1,
+    ignore_negative_samples: Optional[bool] = False,
+    slice_height: Optional[int] = 512,
+    slice_width: Optional[int] = 512,
+    overlap_height_ratio: Optional[float] = 0.2,
+    overlap_width_ratio: Optional[float] = 0.2,
+    min_area_ratio: Optional[float] = 0.1,
     out_ext: Optional[str] = None,
-    verbose: bool = False,
+    verbose: Optional[bool] = False,
+    exif_fix: bool = True,
 ) -> List[Union[Dict, str]]:
     """
-    Slice large images given in a directory, into smaller windows. If out_name is given export sliced images and coco file.
+    Slice large images given in a directory, into smaller windows. If output_dir is given, export sliced images and coco file.
 
     Args:
-        coco_annotation_file_pat (str): Location of the coco annotation file
+        coco_annotation_file_path (str): Location of the coco annotation file
         image_dir (str): Base directory for the images
         output_coco_annotation_file_name (str): File name of the exported coco
-            datatset json.
+            dataset json.
         output_dir (str, optional): Output directory
-        ignore_negative_samples (bool): If True, images without annotations
+        ignore_negative_samples (bool, optional): If True, images without annotations
             are ignored. Defaults to False.
-        slice_height (int): Height of each slice. Default 512.
-        slice_width (int): Width of each slice. Default 512.
-        overlap_height_ratio (float): Fractional overlap in height of each
+        slice_height (int, optional): Height of each slice. Default 512.
+        slice_width (int, optional): Width of each slice. Default 512.
+        overlap_height_ratio (float, optional): Fractional overlap in height of each
             slice (e.g. an overlap of 0.2 for a slice of size 100 yields an
             overlap of 20 pixels). Default 0.2.
-        overlap_width_ratio (float): Fractional overlap in width of each
+        overlap_width_ratio (float, optional): Fractional overlap in width of each
             slice (e.g. an overlap of 0.2 for a slice of size 100 yields an
             overlap of 20 pixels). Default 0.2.
         min_area_ratio (float): If the cropped annotation area to original annotation
@@ -448,7 +441,7 @@ def slice_coco(
         out_ext (str, optional): Extension of saved images. Default is the
             original suffix.
         verbose (bool, optional): Switch to print relevant values to screen.
-            Default 'False'.
+        exif_fix (bool, optional): Whether to apply an EXIF fix to the image.
 
     Returns:
         coco_dict: dict
@@ -465,7 +458,7 @@ def slice_coco(
     sliced_coco_images: List = []
 
     # iterate over images and slice
-    for coco_image in tqdm(coco.images):
+    for idx, coco_image in enumerate(tqdm(coco.images)):
         # get image path
         image_path: str = os.path.join(image_dir, coco_image.file_name)
         # get annotation json list corresponding to selected coco image
@@ -474,7 +467,7 @@ def slice_coco(
             slice_image_result = slice_image(
                 image=image_path,
                 coco_annotation_list=coco_image.annotations,
-                output_file_name=Path(coco_image.file_name).stem,
+                output_file_name=f"{Path(coco_image.file_name).stem}_{idx}",
                 output_dir=output_dir,
                 slice_height=slice_height,
                 slice_width=slice_width,
@@ -483,6 +476,7 @@ def slice_coco(
                 min_area_ratio=min_area_ratio,
                 out_ext=out_ext,
                 verbose=verbose,
+                exif_fix=exif_fix,
             )
             # append slice outputs
             sliced_coco_images.extend(slice_image_result.coco_images)
@@ -501,7 +495,7 @@ def slice_coco(
     return coco_dict, save_path
 
 
-def calc_ratio_and_slice(orientation, slide=1, ratio=0.1):
+def calc_ratio_and_slice(orientation: Literal["vertical", "horizontal", "square"], slide: int = 1, ratio: float = 0.1):
     """
     According to image resolution calculation overlap params
     Args:
@@ -518,6 +512,8 @@ def calc_ratio_and_slice(orientation, slide=1, ratio=0.1):
         slice_row, slice_col, overlap_height_ratio, overlap_width_ratio = slide * 2, slide, ratio, ratio
     elif orientation == "square":
         slice_row, slice_col, overlap_height_ratio, overlap_width_ratio = slide, slide, ratio, ratio
+    else:
+        raise ValueError(f"Invalid orientation: {orientation}. Must be one of 'vertical', 'horizontal', or 'square'.")
 
     return slice_row, slice_col, overlap_height_ratio, overlap_width_ratio  # noqa
 
@@ -557,7 +553,9 @@ def calc_aspect_ratio_orientation(width: int, height: int) -> str:
         return "square"
 
 
-def calc_slice_and_overlap_params(resolution: str, height: int, width: int, orientation: str) -> List:
+def calc_slice_and_overlap_params(
+    resolution: str, height: int, width: int, orientation: str
+) -> Tuple[int, int, int, int]:
     """
     This function calculate according to image resolution slice and overlap params.
     Args:
@@ -596,10 +594,10 @@ def calc_slice_and_overlap_params(resolution: str, height: int, width: int, orie
     x_overlap = int(slice_width * overlap_width_ratio)
     y_overlap = int(slice_height * overlap_height_ratio)
 
-    return x_overlap, y_overlap, slice_width, slice_height  # noqa
+    return x_overlap, y_overlap, slice_width, slice_height
 
 
-def get_resolution_selector(res: str, height: int, width: int):
+def get_resolution_selector(res: str, height: int, width: int) -> Tuple[int, int, int, int]:
     """
 
     Args:
@@ -618,7 +616,7 @@ def get_resolution_selector(res: str, height: int, width: int):
     return x_overlap, y_overlap, slice_width, slice_height
 
 
-def get_auto_slice_params(height: int, width: int):
+def get_auto_slice_params(height: int, width: int) -> Tuple[int, int, int, int]:
     """
     According to Image HxW calculate overlap sliding window and buffer params
     factor is the power value of 2 closest to the image resolution.
@@ -695,7 +693,7 @@ def shift_masks(masks: np.ndarray, offset: Sequence[int], full_shape: Sequence[i
 
     shifted_masks = []
     for mask in masks:
-        mask = Mask(bool_mask=mask, shift_amount=offset, full_shape=full_shape)
+        mask = Mask(segmentation=mask, shift_amount=offset, full_shape=full_shape)
         mask = mask.get_shifted_mask()
         shifted_masks.append(mask.bool_mask)
 
