@@ -1,38 +1,31 @@
-# OBSS SAHI Tool
-# Code written by Fatih C Akyon, 2020.
 # Modified by Sinan O Altinuc, 2020.
 
 import copy
-import logging
 import os
+import random
 import threading
+import warnings
 from collections import Counter, defaultdict
 from dataclasses import dataclass
 from multiprocessing import Pool
 from pathlib import Path
 from threading import Lock, Thread
-from typing import Dict, List, Optional, Set, Union
+from typing import Dict, List, Literal, Optional, Set, Union
 
 import numpy as np
 from tqdm import tqdm
 
+from sahi.logger import logger
 from sahi.utils.file import is_colab, load_json, save_json
 from sahi.utils.shapely import ShapelyAnnotation, box, get_shapely_multipolygon
 
-logger = logging.getLogger(__name__)
-logging.basicConfig(
-    format="%(asctime)s - %(levelname)s - %(name)s -   %(message)s",
-    datefmt="%m/%d/%Y %H:%M:%S",
-    level=os.environ.get("LOGLEVEL", "INFO").upper(),
-)
-
 
 class CocoCategory:
     """
     COCO formatted category.
     """
 
-    def __init__(self, id=None, name=None, supercategory=None):
+    def __init__(self, id: int = 0, name: Optional[str] = None, supercategory: Optional[str] = None):
         self.id = int(id)
         self.name = name
         self.supercategory = supercategory if supercategory else name
@@ -128,7 +121,7 @@ class CocoAnnotation:
             annotation_dict: dict
                 COCO formatted annotation dict (with fields "bbox", "segmentation", "category_id")
         """
-        if annotation_dict.__contains__("segmentation") and not isinstance(annotation_dict["segmentation"], list):
+        if annotation_dict.__contains__("segmentation") and isinstance(annotation_dict["segmentation"], dict):
             has_rle_segmentation = True
             logger.warning(
                 f"Segmentation annotation for id {annotation_dict['id']} is skipped since RLE segmentation format is not supported."
@@ -182,10 +175,10 @@ class CocoAnnotation:
 
     def __init__(
         self,
+        category_id: int,
+        category_name: Optional[str] = None,
         segmentation=None,
-        bbox=None,
-        category_id=None,
-        category_name=None,
+        bbox: Optional[List[int]] = None,
         image_id=None,
         iscrowd=0,
     ):
@@ -218,6 +211,8 @@ class CocoAnnotation:
         if self._segmentation:
             shapely_annotation = ShapelyAnnotation.from_coco_segmentation(segmentation=self._segmentation)
         else:
+            if not bbox:
+                raise TypeError("Coco bounding box not set")
             shapely_annotation = ShapelyAnnotation.from_coco_bbox(bbox=bbox)
         self._shapely_annotation = shapely_annotation
 
@@ -227,7 +222,7 @@ class CocoAnnotation:
         return CocoAnnotation.from_shapely_annotation(
             intersection_shapely_annotation,
             category_id=self.category_id,
-            category_name=self.category_name,
+            category_name=self.category_name or "",
             iscrowd=self.iscrowd,
         )
 
@@ -313,7 +308,8 @@ class CocoAnnotation:
         }
 
     def serialize(self):
-        print(".serialize() is deprectaed, use .json instead")
+        warnings.warn("Use json property instead of serialize method", DeprecationWarning, stacklevel=2)
+        return self.json
 
     def __repr__(self):
         return f"""CocoAnnotation<
@@ -417,8 +413,8 @@ class CocoPrediction(CocoAnnotation):
         self,
         segmentation=None,
         bbox=None,
-        category_id=None,
-        category_name=None,
+        category_id: int = 0,
+        category_name: str = "",
         image_id=None,
         score=None,
         iscrowd=0,
@@ -465,7 +461,7 @@ class CocoPrediction(CocoAnnotation):
         }
 
     def serialize(self):
-        print(".serialize() is deprectaed, use .json instead")
+        warnings.warn("Use json property instead of serialize method", DeprecationWarning, stacklevel=2)
 
     def __repr__(self):
         return f"""CocoPrediction<
@@ -487,9 +483,9 @@ class CocoVidAnnotation(CocoAnnotation):
 
     def __init__(
         self,
-        bbox=None,
-        category_id=None,
-        category_name=None,
+        category_id: int,
+        category_name: str,
+        bbox: List[int],
         image_id=None,
         instance_id=None,
         iscrowd=0,
@@ -566,7 +562,7 @@ class CocoImage:
             width=image_dict["width"],
         )
 
-    def __init__(self, file_name: str, height: int, width: int, id: int = None):
+    def __init__(self, file_name: str, height: int, width: int, id: Optional[int] = None):
         """
         Creates CocoImage object
 
@@ -726,10 +722,10 @@ class CocoVideo:
     def __init__(
         self,
         name: str,
-        id: int = None,
-        fps: float = None,
-        height: int = None,
-        width: int = None,
+        id: Optional[int] = None,
+        fps: Optional[float] = None,
+        height: Optional[int] = None,
+        width: Optional[int] = None,
     ):
         """
         Creates CocoVideo object
@@ -798,12 +794,12 @@ class CocoVideo:
 class Coco:
     def __init__(
         self,
-        name=None,
-        image_dir=None,
-        remapping_dict=None,
-        ignore_negative_samples=False,
-        clip_bboxes_to_img_dims=False,
-        image_id_setting="auto",
+        name: Optional[str] = None,
+        image_dir: Optional[str] = None,
+        remapping_dict: Optional[Dict[int, int]] = None,
+        ignore_negative_samples: bool = False,
+        clip_bboxes_to_img_dims: bool = False,
+        image_id_setting: Literal["auto", "manual"] = "auto",
     ):
         """
         Creates Coco object.
@@ -824,11 +820,11 @@ class Coco:
         """
         if image_id_setting not in ["auto", "manual"]:
             raise ValueError("image_id_setting must be either 'auto' or 'manual'")
-        self.name = name
-        self.image_dir = image_dir
-        self.remapping_dict = remapping_dict
+        self.name: Optional[str] = name
+        self.image_dir: Optional[str] = image_dir
+        self.remapping_dict: Optional[Dict[int, int]] = remapping_dict
         self.ignore_negative_samples = ignore_negative_samples
-        self.categories = []
+        self.categories: List[CocoCategory] = []
         self.images = []
         self._stats = None
         self.clip_bboxes_to_img_dims = clip_bboxes_to_img_dims
@@ -880,7 +876,7 @@ class Coco:
             raise ValueError("image id should be manually set for image_id_setting='manual'")
         self.images.append(image)
 
-    def update_categories(self, desired_name2id, update_image_filenames=False):
+    def update_categories(self, desired_name2id: Dict[str, int], update_image_filenames: bool = False):
         """
         Rearranges category mapping of given COCO object based on given desired_name2id.
         Can also be used to filter some of the categories.
@@ -892,7 +888,7 @@ class Coco:
                 If True, updates coco image file_names with absolute file paths.
         """
         # init vars
-        currentid2desiredid_mapping = {}
+        currentid2desiredid_mapping: Dict[int, Optional[int]] = {}
         updated_coco = Coco(
             name=self.name,
             image_dir=self.image_dir,
@@ -900,9 +896,12 @@ class Coco:
             ignore_negative_samples=self.ignore_negative_samples,
         )
         # create category id mapping (currentid2desiredid_mapping)
-        for coco_category in copy.deepcopy(self.categories):
+        for coco_category in self.categories:
             current_category_id = coco_category.id
             current_category_name = coco_category.name
+            if not current_category_name:
+                logger.warning("no category name provided to update categories")
+                continue
             if current_category_name in desired_name2id.keys():
                 currentid2desiredid_mapping[current_category_id] = desired_name2id[current_category_name]
             else:
@@ -920,7 +919,10 @@ class Coco:
             # update filename to abspath
             file_name_is_abspath = True if os.path.abspath(coco_image.file_name) == coco_image.file_name else False
             if update_image_filenames and not file_name_is_abspath:
-                updated_coco_image.file_name = str(Path(os.path.abspath(self.image_dir)) / coco_image.file_name)
+                if not self.image_dir:
+                    logger.error("image directory not set")
+                else:
+                    updated_coco_image.file_name = str(Path(os.path.abspath(self.image_dir)) / coco_image.file_name)
             # update annotations
             for coco_annotation in coco_image.annotations:
                 current_category_id = coco_annotation.category_id
@@ -934,7 +936,6 @@ class Coco:
             updated_coco.add_image(updated_coco_image)
 
         # overwrite instance
-        self.__class__ = updated_coco.__class__
         self.__dict__ = updated_coco.__dict__
 
     def merge(self, coco, desired_name2id=None, verbose=1):
@@ -1033,7 +1034,7 @@ class Coco:
             raise TypeError("coco_dict_or_path should be a dict or str")
 
         # load coco dict if path is given
-        if type(coco_dict_or_path) == str:
+        if isinstance(coco_dict_or_path, str):
             coco_dict = load_json(coco_dict_or_path)
         else:
             coco_dict = coco_dict_or_path
@@ -1115,6 +1116,9 @@ class Coco:
                     image_id_set.add(image_id)
                 # select annotations of the image
                 annotation_list = image_id_to_annotation_list[image_id]
+                # TODO: coco_annotation_dict is of type CocoAnnotation according to how image_id_to_annotation_list
+                # was created. Either image_id_to_annotation_list is not defined correctly or the following
+                # loop is wrong as it expects a dict.
                 for coco_annotation_dict in annotation_list:
                     # apply category remapping if remapping_dict is provided
                     if coco.remapping_dict is not None:
@@ -1253,7 +1257,8 @@ class Coco:
         Args:
             train_split_rate: float
             numpy_seed: int
-                To fix the numpy seed.
+                random seed. Actually, this doesn't use numpy, but the random package
+                from the standard library, but it is called numpy for compatibility.
 
         Returns:
             result : dict
@@ -1262,13 +1267,11 @@ class Coco:
                     "val_coco": "",
                 }
         """
-        # fix numpy numpy seed
-        np.random.seed(numpy_seed)
-
         # divide images
         num_images = len(self.images)
         shuffled_images = copy.deepcopy(self.images)
-        np.random.shuffle(shuffled_images)
+        random.seed(numpy_seed)
+        random.shuffle(shuffled_images)
         num_train = int(num_images * train_split_rate)
         train_images = shuffled_images[:num_train]
         val_images = shuffled_images[num_train:]
@@ -1291,9 +1294,40 @@ class Coco:
             "val_coco": val_coco,
         }
 
-    def export_as_yolov5(self, output_dir, train_split_rate=1, numpy_seed=0, mp=False, disable_symlink=False):
+    def export_as_yolov5(
+        self,
+        output_dir: Union[str, Path],
+        train_split_rate: float = 1.0,
+        numpy_seed: int = 0,
+        mp: bool = False,
+        disable_symlink: bool = False,
+    ):
         """
-        Exports current COCO dataset in ultralytics/yolov5 format.
+        Deprecated. Please use export_as_yolo instead.
+        Calls export_as_yolo with the same arguments.
+        """
+        warnings.warn(
+            "export_as_yolov5 is deprecated. Please use export_as_yolo instead.",
+            DeprecationWarning,
+        )
+        self.export_as_yolo(
+            output_dir=output_dir,
+            train_split_rate=train_split_rate,
+            numpy_seed=numpy_seed,
+            mp=mp,
+            disable_symlink=disable_symlink,
+        )
+
+    def export_as_yolo(
+        self,
+        output_dir: Union[str, Path],
+        train_split_rate: float = 1.0,
+        numpy_seed: int = 0,
+        mp: bool = False,
+        disable_symlink: bool = False,
+    ):
+        """
+        Exports current COCO dataset in ultralytics/yolo format.
         Creates train val folders with image symlinks and txt files and a data yaml file.
 
         Args:
@@ -1314,9 +1348,7 @@ class Coco:
         try:
             import yaml
         except ImportError:
-            raise ImportError(
-                'Please run "pip install -U pyyaml" ' "to install yaml first for yolov5 formatted exporting."
-            )
+            raise ImportError('Please run "pip install -U pyyaml" to install yaml first for yolo formatted exporting.')
 
         # set split_mode
         if 0 < train_split_rate and train_split_rate < 1:
@@ -1355,7 +1387,7 @@ class Coco:
 
         # create image symlinks and annotation txts
         if split_mode in ["TRAINVAL", "TRAIN"]:
-            export_yolov5_images_and_txts_from_coco_object(
+            export_yolo_images_and_txts_from_coco_object(
                 output_dir=train_dir,
                 coco=train_coco,
                 ignore_negative_samples=self.ignore_negative_samples,
@@ -1363,7 +1395,7 @@ class Coco:
                 disable_symlink=disable_symlink,
             )
         if split_mode in ["TRAINVAL", "VAL"]:
-            export_yolov5_images_and_txts_from_coco_object(
+            export_yolo_images_and_txts_from_coco_object(
                 output_dir=val_dir,
                 coco=val_coco,
                 ignore_negative_samples=self.ignore_negative_samples,
@@ -1382,7 +1414,7 @@ class Coco:
         with open(yaml_path, "w") as outfile:
             yaml.dump(data, outfile, default_flow_style=None)
 
-    def get_subsampled_coco(self, subsample_ratio: int = 2, category_id: int = None):
+    def get_subsampled_coco(self, subsample_ratio: int = 2, category_id: Optional[int] = None):
         """
         Subsamples images with subsample_ratio and returns as sahi.utils.coco.Coco object.
 
@@ -1405,9 +1437,9 @@ class Coco:
         if category_id is not None:
             # get images that contain given category id
             images_that_contain_category: List[CocoImage] = []
+            annotation: CocoAnnotation
             for image in self.images:
                 category_id_to_contains = defaultdict(lambda: 0)
-                annotation: CocoAnnotation
                 for annotation in image.annotations:
                     category_id_to_contains[annotation.category_id] = 1
                 if category_id_to_contains[category_id]:
@@ -1425,7 +1457,6 @@ class Coco:
             images_that_doesnt_contain_category: List[CocoImage] = []
             for image in self.images:
                 category_id_to_contains = defaultdict(lambda: 0)
-                annotation: CocoAnnotation
                 for annotation in image.annotations:
                     category_id_to_contains[annotation.category_id] = 1
                 if category_id_to_contains[category_id]:
@@ -1451,7 +1482,7 @@ class Coco:
 
         return subsampled_coco
 
-    def get_upsampled_coco(self, upsample_ratio: int = 2, category_id: int = None):
+    def get_upsampled_coco(self, upsample_ratio: int = 2, category_id: Optional[int] = None):
         """
         Upsamples images with upsample_ratio and returns as sahi.utils.coco.Coco object.
 
@@ -1573,7 +1604,7 @@ class Coco:
         return coco
 
 
-def export_yolov5_images_and_txts_from_coco_object(
+def export_yolo_images_and_txts_from_coco_object(
     output_dir, coco, ignore_negative_samples=False, mp=False, disable_symlink=False
 ):
     """
@@ -1592,7 +1623,7 @@ def export_yolov5_images_and_txts_from_coco_object(
         disable_symlink: bool
             If True, symlinks are not created. Instead images are copied.
     """
-    logger.info("generating image symlinks and annotation files for yolov5..."),
+    logger.info("generating image symlinks and annotation files for yolo...")
     # symlink is not supported in colab
     if is_colab() and not disable_symlink:
         logger.warning("symlink is not supported in colab, disabling it...")
@@ -1604,21 +1635,21 @@ def export_yolov5_images_and_txts_from_coco_object(
                 for coco_image in coco.images
             ]
             pool.starmap(
-                export_single_yolov5_image_and_corresponding_txt,
+                export_single_yolo_image_and_corresponding_txt,
                 tqdm(args, total=len(args)),
             )
     else:
         for coco_image in tqdm(coco.images):
-            export_single_yolov5_image_and_corresponding_txt(
+            export_single_yolo_image_and_corresponding_txt(
                 coco_image, coco.image_dir, output_dir, ignore_negative_samples, disable_symlink
             )
 
 
-def export_single_yolov5_image_and_corresponding_txt(
+def export_single_yolo_image_and_corresponding_txt(
     coco_image, coco_image_dir, output_dir, ignore_negative_samples=False, disable_symlink=False
 ):
     """
-    Generates yolov5 formatted image symlink and annotation txt file.
+    Generates YOLO formatted image symlink and annotation txt file.
 
     Args:
         coco_image: sahi.utils.coco.CocoImage
@@ -1652,7 +1683,7 @@ def export_single_yolov5_image_and_corresponding_txt(
         coco_image_path = os.path.abspath(coco_image.file_name)
     else:
         if coco_image_dir is None:
-            raise ValueError("You have to specify image_dir of Coco object for yolov5 conversion.")
+            raise ValueError("You have to specify image_dir of Coco object for yolo conversion.")
 
         coco_image_path = os.path.abspath(str(Path(coco_image_dir) / coco_image.file_name))
 
@@ -1715,9 +1746,9 @@ def update_categories(desired_name2id: dict, coco_dict: dict) -> dict:
     Returns:
     ---------
         coco_target : dict
-            COCO dict with updated/filtred categories.
+            COCO dict with updated/filtered categories.
     """
-    # so that original variable doesnt get affected
+    # so that original variable doesn't get affected
     coco_source = copy.deepcopy(coco_dict)
 
     # init target coco dict
@@ -1784,7 +1815,7 @@ def update_categories_from_file(desired_name2id: dict, coco_path: str, save_path
     save_json(coco_target, save_path)
 
 
-def merge(coco_dict1: dict, coco_dict2: dict, desired_name2id: dict = None) -> dict:
+def merge(coco_dict1: dict, coco_dict2: dict, desired_name2id: Optional[dict] = None) -> dict:
     """
     Combines 2 coco formatted annotations dicts, and returns the combined coco dict.
 
@@ -1909,9 +1940,7 @@ def merge_from_file(coco_path1: str, coco_path2: str, save_path: str):
     save_json(merged_coco_dict, save_path)
 
 
-def get_imageid2annotationlist_mapping(
-    coco_dict: dict,
-) -> Dict[int, List[CocoAnnotation]]:
+def get_imageid2annotationlist_mapping(coco_dict: dict) -> Dict[int, List[CocoAnnotation]]:
     """
     Get image_id to annotationlist mapping for faster indexing.
 
@@ -1939,7 +1968,7 @@ def get_imageid2annotationlist_mapping(
         }
     """
     image_id_to_annotation_list: Dict = defaultdict(list)
-    print("indexing coco dataset annotations...")
+    logger.debug("indexing coco dataset annotations...")
     for annotation in coco_dict["annotations"]:
         image_id = annotation["image_id"]
         image_id_to_annotation_list[image_id].append(annotation)
@@ -2143,7 +2172,7 @@ class DatasetClassCounts:
     total_images: int
 
     def frequencies(self):
-        """calculates the frequenct of images that contain each category"""
+        """calculates the frequency of images that contain each category"""
         return {cid: count / self.total_images for cid, count in self.counts.items()}
 
     def __add__(self, o):
@@ -2220,7 +2249,7 @@ class CocoVid:
 
             self.add_category(CocoCategory.from_coco_category(coco_category))
 
-    def add_category(self, category):
+    def add_category(self, category: CocoCategory):
         """
         Adds category to this CocoVid instance
 
@@ -2228,8 +2257,8 @@ class CocoVid:
             category: CocoCategory
         """
 
-        if type(category) != CocoCategory:
-            raise TypeError("category must be a CocoCategory instance")
+        if not isinstance(category, CocoCategory):
+            raise TypeError("category must be a CocoCategory instance")  # type: ignore
         self.categories.append(category)
 
     @property
@@ -2246,7 +2275,7 @@ class CocoVid:
             category_mapping[category.id] = category.name
         return category_mapping
 
-    def add_video(self, video):
+    def add_video(self, video: CocoVideo):
         """
         Adds video to this CocoVid instance
 
@@ -2254,8 +2283,8 @@ class CocoVid:
             video: CocoVideo
         """
 
-        if type(video) != CocoVideo:
-            raise TypeError("video must be a CocoVideo instance")
+        if not isinstance(video, CocoVideo):
+            raise TypeError("video must be a CocoVideo instance")  # type: ignore
         self.videos.append(video)
 
     @property
@@ -2302,7 +2331,9 @@ class CocoVid:
         return coco_dict
 
 
-def remove_invalid_coco_results(result_list_or_path: Union[List, str], dataset_dict_or_path: Union[Dict, str] = None):
+def remove_invalid_coco_results(
+    result_list_or_path: Union[List, str], dataset_dict_or_path: Union[Dict, str, None] = None
+):
     """
     Removes invalid predictions from coco result such as:
         - negative bbox value
@@ -2319,7 +2350,7 @@ def remove_invalid_coco_results(result_list_or_path: Union[List, str], dataset_d
     elif isinstance(result_list_or_path, list):
         result_list = result_list_or_path
     else:
-        raise TypeError('incorrect type for "result_list_or_path"')
+        raise TypeError('incorrect type for "result_list_or_path"')  # type: ignore
 
     # prepare image info from coco dataset
     if dataset_dict_or_path is not None:
@@ -2328,7 +2359,7 @@ def remove_invalid_coco_results(result_list_or_path: Union[List, str], dataset_d
         elif isinstance(dataset_dict_or_path, dict):
             dataset_dict = dataset_dict_or_path
         else:
-            raise TypeError('incorrect type for "dataset_dict"')
+            raise TypeError('incorrect type for "dataset_dict"')  # type: ignore
         image_id_to_height = {}
         image_id_to_width = {}
         for coco_image in dataset_dict["images"]:
@@ -2361,14 +2392,40 @@ def remove_invalid_coco_results(result_list_or_path: Union[List, str], dataset_d
 
 def export_coco_as_yolov5(
     output_dir: str,
-    train_coco: Coco = None,
-    val_coco: Coco = None,
+    train_coco: Optional[Coco] = None,
+    val_coco: Optional[Coco] = None,
     train_split_rate: float = 0.9,
     numpy_seed=0,
     disable_symlink=False,
 ):
     """
-    Exports current COCO dataset in ultralytics/yolov5 format.
+    Deprecated. Please use export_coco_as_yolo instead.
+    Calls export_coco_as_yolo with the same arguments.
+    """
+    warnings.warn(
+        "export_coco_as_yolov5 is deprecated. Please use export_coco_as_yolo instead.",
+        DeprecationWarning,
+    )
+    export_coco_as_yolo(
+        output_dir=output_dir,
+        train_coco=train_coco,
+        val_coco=val_coco,
+        train_split_rate=train_split_rate,
+        numpy_seed=numpy_seed,
+        disable_symlink=disable_symlink,
+    )
+
+
+def export_coco_as_yolo(
+    output_dir: str,
+    train_coco: Optional[Coco] = None,
+    val_coco: Optional[Coco] = None,
+    train_split_rate: float = 0.9,
+    numpy_seed=0,
+    disable_symlink=False,
+):
+    """
+    Exports current COCO dataset in ultralytics/YOLO format.
     Creates train val folders with image symlinks and txt files and a data yaml file.
 
     Args:
@@ -2387,12 +2444,12 @@ def export_coco_as_yolov5(
 
     Returns:
         yaml_path: str
-            Path for the exported yolov5 data.yml
+            Path for the exported YOLO data.yml
     """
     try:
         import yaml
     except ImportError:
-        raise ImportError('Please run "pip install -U pyyaml" ' "to install yaml first for yolov5 formatted exporting.")
+        raise ImportError('Please run "pip install -U pyyaml" to install yaml first for YOLO formatted exporting.')
 
     # set split_mode
     if train_coco and not val_coco:
@@ -2422,14 +2479,15 @@ def export_coco_as_yolov5(
     val_dir.mkdir(parents=True, exist_ok=True)  # create dir
 
     # create image symlinks and annotation txts
-    export_yolov5_images_and_txts_from_coco_object(
+    export_yolo_images_and_txts_from_coco_object(
         output_dir=train_dir,
         coco=train_coco,
         ignore_negative_samples=train_coco.ignore_negative_samples,
         mp=False,
         disable_symlink=disable_symlink,
     )
-    export_yolov5_images_and_txts_from_coco_object(
+    assert val_coco, "Validation Coco object not set"
+    export_yolo_images_and_txts_from_coco_object(
         output_dir=val_dir,
         coco=val_coco,
         ignore_negative_samples=val_coco.ignore_negative_samples,
@@ -2455,7 +2513,27 @@ def export_coco_as_yolov5_via_yml(
     yml_path: str, output_dir: str, train_split_rate: float = 0.9, numpy_seed=0, disable_symlink=False
 ):
     """
-    Exports current COCO dataset in ultralytics/yolov5 format.
+    Deprecated. Please use export_coco_as_yolo_via_yml instead.
+    Calls export_coco_as_yolo_via_yml with the same arguments.
+    """
+    warnings.warn(
+        "export_coco_as_yolov5_via_yml is deprecated. Please use export_coco_as_yolo_via_yml instead.",
+        DeprecationWarning,
+    )
+    export_coco_as_yolo_via_yml(
+        yml_path=yml_path,
+        output_dir=output_dir,
+        train_split_rate=train_split_rate,
+        numpy_seed=numpy_seed,
+        disable_symlink=disable_symlink,
+    )
+
+
+def export_coco_as_yolo_via_yml(
+    yml_path: str, output_dir: str, train_split_rate: float = 0.9, numpy_seed=0, disable_symlink=False
+):
+    """
+    Exports current COCO dataset in ultralytics/YOLO format.
     Creates train val folders with image symlinks and txt files and a data yaml file.
     Uses a yml file as input.
 
@@ -2477,12 +2555,12 @@ def export_coco_as_yolov5_via_yml(
 
     Returns:
         yaml_path: str
-            Path for the exported yolov5 data.yml
+            Path for the exported YOLO data.yml
     """
     try:
         import yaml
     except ImportError:
-        raise ImportError('Please run "pip install -U pyyaml" ' "to install yaml first for yolov5 formatted exporting.")
+        raise ImportError('Please run "pip install -U pyyaml" to install yaml first for YOLO formatted exporting.')
 
     with open(yml_path, "r") as stream:
         config_dict = yaml.safe_load(stream)
@@ -2503,7 +2581,7 @@ def export_coco_as_yolov5_via_yml(
     else:
         val_coco = None
 
-    yaml_path = export_coco_as_yolov5(
+    yaml_path = export_coco_as_yolo(
         output_dir=output_dir,
         train_coco=train_coco,
         val_coco=val_coco,
