@@ -1,25 +1,25 @@
-# OBSS SAHI Tool
-# Code written by Fatih C Akyon, 2020.
-
 import copy
 import os
 import random
 import time
-from typing import List, Optional, Union
+from io import BytesIO
+from typing import Generator, List, Optional, Tuple, Union
 
 import cv2
 import numpy as np
 import requests
-from PIL import Image
+from PIL import Image, ImageOps
 
+from sahi.logger import logger
 from sahi.utils.file import Path
 
-IMAGE_EXTENSIONS = [".jpg", ".jpeg", ".png", ".tiff", ".bmp"]
+IMAGE_EXTENSIONS_LOSSY = [".jpg", ".jpeg"]
+IMAGE_EXTENSIONS_LOSSLESS = [".png", ".tif", ".tiff", ".bmp"]
+IMAGE_EXTENSIONS = IMAGE_EXTENSIONS_LOSSY + IMAGE_EXTENSIONS_LOSSLESS
 VIDEO_EXTENSIONS = [".mp4", ".mkv", ".flv", ".avi", ".ts", ".mpg", ".mov", "wmv"]
 
 
 class Colors:
-    # color palette
     def __init__(self):
         hex = (
             "FF3838",
@@ -43,16 +43,38 @@ class Colors:
             "FF95C8",
             "FF37C7",
         )
-        self.palette = [self.hex2rgb("#" + c) for c in hex]
+        self.palette = [self.hex_to_rgb("#" + c) for c in hex]
         self.n = len(self.palette)
 
-    def __call__(self, i, bgr=False):
-        c = self.palette[int(i) % self.n]
-        return (c[2], c[1], c[0]) if bgr else c
+    def __call__(self, ind, bgr: bool = False):
+        """
+        Convert an index to a color code.
+
+        Args:
+            ind (int): The index to convert.
+            bgr (bool, optional): Whether to return the color code in BGR format. Defaults to False.
+
+        Returns:
+            tuple: The color code in RGB or BGR format, depending on the value of `bgr`.
+        """
+        color_codes = self.palette[int(ind) % self.n]
+        return (color_codes[2], color_codes[1], color_codes[0]) if bgr else color_codes
 
     @staticmethod
-    def hex2rgb(h):  # rgb order
-        return tuple(int(h[1 + i : 1 + i + 2], 16) for i in (0, 2, 4))
+    def hex_to_rgb(hex_code):
+        """
+        Converts a hexadecimal color code to RGB format.
+
+        Args:
+            hex_code (str): The hexadecimal color code to convert.
+
+        Returns:
+            tuple: A tuple representing the RGB values in the order (R, G, B).
+        """
+        rgb = []
+        for i in (0, 2, 4):
+            rgb.append(int(hex_code[1 + i : 1 + i + 2], 16))
+        return tuple(rgb)
 
 
 def crop_object_predictions(
@@ -63,23 +85,25 @@ def crop_object_predictions(
     export_format: str = "png",
 ):
     """
-    Crops bounding boxes over the source image and exports it to output folder.
-    Arguments:
-        object_predictions: a list of prediction.ObjectPrediction
-        output_dir: directory for resulting visualization to be exported
-        file_name: exported file will be saved as: output_dir+file_name+".png"
-        export_format: can be specified as 'jpg' or 'png'
+    Crops bounding boxes over the source image and exports it to the output folder.
+
+    Args:
+        image (np.ndarray): The source image to crop bounding boxes from.
+        object_prediction_list: A list of object predictions.
+        output_dir (str): The directory where the resulting visualizations will be exported. Defaults to an empty string.
+        file_name (str): The name of the exported file. The exported file will be saved as `output_dir + file_name + ".png"`. Defaults to "prediction_visual".
+        export_format (str): The format of the exported file. Can be specified as 'jpg' or 'png'. Defaults to "png".
     """
     # create output folder if not present
     Path(output_dir).mkdir(parents=True, exist_ok=True)
     # add bbox and mask to image if present
     for ind, object_prediction in enumerate(object_prediction_list):
-        # deepcopy object_prediction_list so that original is not altered
+        # deepcopy object_prediction_list so that the original is not altered
         object_prediction = object_prediction.deepcopy()
         bbox = object_prediction.bbox.to_xyxy()
         category_id = object_prediction.category.id
         # crop detections
-        # deepcopy crops so that original is not altered
+        # deepcopy crops so that the original is not altered
         cropped_img = copy.deepcopy(
             image[
                 int(bbox[1]) : int(bbox[3]),
@@ -96,10 +120,15 @@ def crop_object_predictions(
 
 def convert_image_to(read_path, extension: str = "jpg", grayscale: bool = False):
     """
-    Reads image from path and saves as given extension.
+    Reads an image from the given path and saves it with the specified extension.
+
+    Args:
+        read_path (str): The path to the image file.
+        extension (str, optional): The desired file extension for the saved image. Defaults to "jpg".
+        grayscale (bool, optional): Whether to convert the image to grayscale. Defaults to False.
     """
     image = cv2.imread(read_path)
-    pre, ext = os.path.splitext(read_path)
+    pre, _ = os.path.splitext(read_path)
     if grayscale:
         image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
         pre = pre + "_gray"
@@ -108,27 +137,45 @@ def convert_image_to(read_path, extension: str = "jpg", grayscale: bool = False)
 
 
 def read_large_image(image_path: str):
+    """
+    Reads a large image from the specified image path.
+
+    Args:
+        image_path (str): The path to the image file.
+
+    Returns:
+        tuple: A tuple containing the image data and a flag indicating whether cv2 was used to read the image.
+            The image data is a numpy array representing the image in RGB format.
+            The flag is True if cv2 was used, False otherwise.
+    """
     use_cv2 = True
     # read image, cv2 fails on large files
     try:
         # convert to rgb (cv2 reads in bgr)
         img_cv2 = cv2.imread(image_path, 1)
         image0 = cv2.cvtColor(img_cv2, cv2.COLOR_BGR2RGB)
-    except:
+    except Exception as e:
+        logger.error(f"OpenCV failed reading image with error {e}, trying skimage instead")
         try:
             import skimage.io
         except ImportError:
             raise ImportError(
-                'Please run "pip install -U scikit-image" ' "to install scikit-image first for large image handling."
+                'Please run "pip install -U scikit-image" to install scikit-image first for large image handling.'
             )
         image0 = skimage.io.imread(image_path, as_grey=False).astype(np.uint8)  # [::-1]
         use_cv2 = False
     return image0, use_cv2
 
 
-def read_image(image_path: str):
+def read_image(image_path: str) -> np.ndarray:
     """
-    Loads image as numpy array from given path.
+    Loads image as a numpy array from the given path.
+
+    Args:
+        image_path (str): The path to the image file.
+
+    Returns:
+        numpy.ndarray: The loaded image as a numpy array.
     """
     # read image
     image = cv2.imread(image_path)
@@ -137,12 +184,17 @@ def read_image(image_path: str):
     return image
 
 
-def read_image_as_pil(image: Union[Image.Image, str, np.ndarray], exif_fix: bool = False):
+def read_image_as_pil(image: Union[Image.Image, str, np.ndarray], exif_fix: bool = True) -> Image.Image:
     """
     Loads an image as PIL.Image.Image.
 
     Args:
-        image : Can be image path or url (str), numpy image (np.ndarray) or PIL.Image
+        image (Union[Image.Image, str, np.ndarray]): The image to be loaded. It can be an image path or URL (str),
+            a numpy image (np.ndarray), or a PIL.Image object.
+        exif_fix (bool, optional): Whether to apply an EXIF fix to the image. Defaults to False.
+
+    Returns:
+        PIL.Image.Image: The loaded image as a PIL.Image object.
     """
     # https://stackoverflow.com/questions/56174099/how-to-load-images-larger-than-max-image-pixels-with-pil
     Image.MAX_IMAGE_PIXELS = None
@@ -153,11 +205,12 @@ def read_image_as_pil(image: Union[Image.Image, str, np.ndarray], exif_fix: bool
         # read image if str image path is provided
         try:
             image_pil = Image.open(
-                requests.get(image, stream=True).raw if str(image).startswith("http") else image
+                BytesIO(requests.get(image, stream=True).content) if str(image).startswith("http") else image
             ).convert("RGB")
             if exif_fix:
-                image_pil = exif_transpose(image_pil)
-        except:  # handle large/tiff image reading
+                ImageOps.exif_transpose(image_pil, in_place=True)
+        except Exception as e:  # handle large/tiff image reading
+            logger.error(f"PIL failed reading image with error {e}, trying skimage instead")
             try:
                 import skimage.io
             except ImportError:
@@ -182,7 +235,11 @@ def read_image_as_pil(image: Union[Image.Image, str, np.ndarray], exif_fix: bool
 
 def select_random_color():
     """
-    Selects random color.
+    Selects a random color from a predefined list of colors.
+
+    Returns:
+        list: A list representing the RGB values of the selected color.
+
     """
     colors = [
         [0, 255, 0],
@@ -200,9 +257,16 @@ def select_random_color():
     return colors[random.randrange(0, 10)]
 
 
-def apply_color_mask(image: np.ndarray, color: tuple):
+def apply_color_mask(image: np.ndarray, color: Tuple[int, int, int]):
     """
     Applies color mask to given input image.
+
+    Args:
+        image (np.ndarray): The input image to apply the color mask to.
+        color (tuple): The RGB color tuple to use for the mask.
+
+    Returns:
+        np.ndarray: The resulting image with the applied color mask.
     """
     r = np.zeros_like(image).astype(np.uint8)
     g = np.zeros_like(image).astype(np.uint8)
@@ -219,7 +283,7 @@ def get_video_reader(
     frame_skip_interval: int,
     export_visual: bool = False,
     view_visual: bool = False,
-):
+) -> Tuple[Generator[Image.Image, None, None], Optional[cv2.VideoWriter], str, int]:
     """
     Creates OpenCV video capture object from given video file path.
 
@@ -245,12 +309,14 @@ def get_video_reader(
         num_frames /= frame_skip_interval + 1
         num_frames = int(num_frames)
 
-    def read_video_frame(video_capture, frame_skip_interval):
+    def read_video_frame(video_capture, frame_skip_interval) -> Generator[Image.Image, None, None]:
         if view_visual:
-            cv2.imshow("Prediction of {}".format(str(video_file_name)), cv2.WINDOW_AUTOSIZE)
+            window_name = "Prediction of {}".format(str(video_file_name))
+            cv2.namedWindow(window_name, cv2.WINDOW_AUTOSIZE)
+            default_image = np.zeros((480, 640, 3), dtype=np.uint8)
+            cv2.imshow(window_name, default_image)
 
             while video_capture.isOpened:
-
                 frame_num = video_capture.get(cv2.CAP_PROP_POS_FRAMES)
                 video_capture.set(cv2.CAP_PROP_POS_FRAMES, frame_num + frame_skip_interval)
 
@@ -276,7 +342,7 @@ def get_video_reader(
                 if not ret:
                     print("\n=========================== Video Ended ===========================")
                     break
-                yield Image.fromarray(frame)
+                yield Image.fromarray(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
 
         else:
             while video_capture.isOpened:
@@ -287,7 +353,7 @@ def get_video_reader(
                 if not ret:
                     print("\n=========================== Video Ended ===========================")
                     break
-                yield Image.fromarray(frame)
+                yield Image.fromarray(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
 
     if export_visual:
         # get video properties and create VideoWriter object
@@ -303,7 +369,7 @@ def get_video_reader(
         w = int(video_capture.get(cv2.CAP_PROP_FRAME_WIDTH))
         h = int(video_capture.get(cv2.CAP_PROP_FRAME_HEIGHT))
         size = (w, h)
-        fourcc = cv2.VideoWriter_fourcc(*"mp4v")
+        fourcc = cv2.VideoWriter_fourcc(*"mp4v")  # pyright: ignore[reportAttributeAccessIssue]
         video_writer = cv2.VideoWriter(os.path.join(save_dir, video_file_name), fourcc, fps, size)
     else:
         video_writer = None
@@ -316,10 +382,10 @@ def visualize_prediction(
     boxes: List[List],
     classes: List[str],
     masks: Optional[List[np.ndarray]] = None,
-    rect_th: float = None,
-    text_size: float = None,
-    text_th: float = None,
-    color: tuple = None,
+    rect_th: Optional[int] = None,
+    text_size: Optional[float] = None,
+    text_th: Optional[int] = None,
+    color: Optional[tuple] = None,
     hide_labels: bool = False,
     output_dir: Optional[str] = None,
     file_name: Optional[str] = "prediction_visual",
@@ -327,6 +393,22 @@ def visualize_prediction(
     """
     Visualizes prediction classes, bounding boxes over the source image
     and exports it to output folder.
+
+    Args:
+        image (np.ndarray): The source image.
+        boxes (List[List]): List of bounding boxes coordinates.
+        classes (List[str]): List of class labels corresponding to each bounding box.
+        masks (Optional[List[np.ndarray]], optional): List of masks corresponding to each bounding box. Defaults to None.
+        rect_th (int, optional): Thickness of the bounding box rectangle. Defaults to None.
+        text_size (float, optional): Size of the text for class labels. Defaults to None.
+        text_th (int, optional): Thickness of the text for class labels. Defaults to None.
+        color (tuple, optional): Color of the bounding box and text. Defaults to None.
+        hide_labels (bool, optional): Whether to hide the class labels. Defaults to False.
+        output_dir (Optional[str], optional): Output directory to save the visualization. Defaults to None.
+        file_name (Optional[str], optional): File name for the saved visualization. Defaults to "prediction_visual".
+
+    Returns:
+        dict: A dictionary containing the visualized image and the elapsed time for the visualization process.
     """
     elapsed_time = time.time()
     # deepcopy image so that original is not altered
@@ -344,7 +426,9 @@ def visualize_prediction(
     text_size = text_size or rect_th / 3
 
     # add masks to image if present
-    if masks is not None:
+    if masks is not None and color is None:
+        logger.error("Cannot add mask, no color tuple given")
+    elif masks is not None and color is not None:
         for mask in masks:
             # deepcopy mask so that original is not altered
             mask = copy.deepcopy(mask)
@@ -353,37 +437,45 @@ def visualize_prediction(
             image = cv2.addWeighted(image, 1, rgb_mask, 0.6, 0)
 
     # add bboxes to image if present
-    for i in range(len(boxes)):
+    for box_indice in range(len(boxes)):
         # deepcopy boxso that original is not altered
-        box = copy.deepcopy(boxes[i])
-        class_ = classes[i]
+        box = copy.deepcopy(boxes[box_indice])
+        class_ = classes[box_indice]
 
         # set color
         if colors is not None:
-            color = colors(class_)
+            mycolor = colors(class_)
+        elif color is not None:
+            mycolor = color
+        else:
+            logger.error("color cannot be defined")
+            continue
+
         # set bbox points
-        p1, p2 = (int(box[0]), int(box[1])), (int(box[2]), int(box[3]))
+        point1, point2 = [int(box[0]), int(box[1])], [int(box[2]), int(box[3])]
         # visualize boxes
         cv2.rectangle(
             image,
-            p1,
-            p2,
-            color=color,
+            point1,
+            point2,
+            color=mycolor,
             thickness=rect_th,
         )
 
         if not hide_labels:
             # arange bounding box text location
             label = f"{class_}"
-            w, h = cv2.getTextSize(label, 0, fontScale=text_size, thickness=text_th)[0]  # label width, height
-            outside = p1[1] - h - 3 >= 0  # label fits outside box
-            p2 = p1[0] + w, p1[1] - h - 3 if outside else p1[1] + h + 3
+            box_width, box_height = cv2.getTextSize(label, 0, fontScale=text_size, thickness=text_th)[
+                0
+            ]  # label width, height
+            outside = point1[1] - box_height - 3 >= 0  # label fits outside box
+            point2 = point1[0] + box_width, point1[1] - box_height - 3 if outside else point1[1] + box_height + 3
             # add bounding box text
-            cv2.rectangle(image, p1, p2, color, -1, cv2.LINE_AA)  # filled
+            cv2.rectangle(image, point1, point2, color or (0, 0, 0), -1, cv2.LINE_AA)  # filled
             cv2.putText(
                 image,
                 label,
-                (p1[0], p1[1] - 2 if outside else p1[1] + h + 2),
+                (point1[0], point1[1] - 2 if outside else point1[1] + box_height + 2),
                 0,
                 text_size,
                 (255, 255, 255),
@@ -393,7 +485,7 @@ def visualize_prediction(
         # create output folder if not present
         Path(output_dir).mkdir(parents=True, exist_ok=True)
         # save inference result
-        save_path = os.path.join(output_dir, file_name + ".png")
+        save_path = os.path.join(output_dir, (file_name or "unknown") + ".png")
         cv2.imwrite(save_path, cv2.cvtColor(image, cv2.COLOR_RGB2BGR))
 
     elapsed_time = time.time() - elapsed_time
@@ -401,22 +493,23 @@ def visualize_prediction(
 
 
 def visualize_object_predictions(
-    image: np.array,
+    image: np.ndarray,
     object_prediction_list,
-    rect_th: int = None,
-    text_size: float = None,
-    text_th: float = None,
-    color: tuple = None,
+    rect_th: Optional[int] = None,
+    text_size: Optional[float] = None,
+    text_th: Optional[int] = None,
+    color: Optional[tuple] = None,
     hide_labels: bool = False,
     hide_conf: bool = False,
     output_dir: Optional[str] = None,
-    file_name: str = "prediction_visual",
-    export_format: str = "png",
+    file_name: Optional[str] = "prediction_visual",
+    export_format: Optional[str] = "png",
 ):
     """
     Visualizes prediction category names, bounding boxes over the source image
     and exports it to output folder.
-    Arguments:
+
+    Args:
         object_prediction_list: a list of prediction.ObjectPrediction
         rect_th: rectangle thickness
         text_size: size of the category name over box
@@ -443,79 +536,101 @@ def visualize_object_predictions(
     # set text_size for category names
     text_size = text_size or rect_th / 3
 
-    # add masks to image if present
+    # add masks or obb polygons to image if present
     for object_prediction in object_prediction_list:
         # deepcopy object_prediction_list so that original is not altered
         object_prediction = object_prediction.deepcopy()
-        # visualize masks if present
-        if object_prediction.mask is not None:
-            # deepcopy mask so that original is not altered
-            mask = object_prediction.mask.bool_mask
-            # set color
-            if colors is not None:
-                color = colors(object_prediction.category.id)
-            # draw mask
-            rgb_mask = apply_color_mask(mask, color)
-            image = cv2.addWeighted(image, 1, rgb_mask, 0.6, 0)
-
-    # add bboxes to image if present
-    for object_prediction in object_prediction_list:
-        # deepcopy object_prediction_list so that original is not altered
-        object_prediction = object_prediction.deepcopy()
-
-        bbox = object_prediction.bbox.to_xyxy()
-        category_name = object_prediction.category.name
-        score = object_prediction.score.value
-
+        # arange label to be displayed
+        label = f"{object_prediction.category.name}"
+        if not hide_conf:
+            label += f" {object_prediction.score.value:.2f}"
         # set color
         if colors is not None:
             color = colors(object_prediction.category.id)
-        # set bbox points
-        p1, p2 = (int(bbox[0]), int(bbox[1])), (int(bbox[2]), int(bbox[3]))
-        # visualize boxes
-        cv2.rectangle(
-            image,
-            p1,
-            p2,
-            color=color,
-            thickness=rect_th,
-        )
+        # visualize masks or obb polygons if present
+        has_mask = object_prediction.mask is not None
+        is_obb_pred = False
+        if has_mask:
+            segmentation = object_prediction.mask.segmentation
+            if len(segmentation) == 1 and len(segmentation[0]) == 8:
+                is_obb_pred = True
 
-        if not hide_labels:
-            # arange bounding box text location
-            label = f"{category_name}"
+            if is_obb_pred:
+                points = np.array(segmentation).reshape((-1, 1, 2)).astype(np.int32)
+                cv2.polylines(image, [points], isClosed=True, color=color or (0, 0, 0), thickness=rect_th)
 
-            if not hide_conf:
-                label += f" {score:.2f}"
+                if not hide_labels:
+                    lowest_point = points[points[:, :, 1].argmax()][0]
+                    box_width, box_height = cv2.getTextSize(label, 0, fontScale=text_size, thickness=text_th)[0]
+                    outside = lowest_point[1] - box_height - 3 >= 0
+                    text_bg_point1 = (
+                        lowest_point[0],
+                        lowest_point[1] - box_height - 3 if outside else lowest_point[1] + 3,
+                    )
+                    text_bg_point2 = (lowest_point[0] + box_width, lowest_point[1])
+                    cv2.rectangle(
+                        image, text_bg_point1, text_bg_point2, color or (0, 0, 0), thickness=-1, lineType=cv2.LINE_AA
+                    )
+                    cv2.putText(
+                        image,
+                        label,
+                        (lowest_point[0], lowest_point[1] - 2 if outside else lowest_point[1] + box_height + 2),
+                        0,
+                        text_size,
+                        (255, 255, 255),
+                        thickness=text_th,
+                    )
+            else:
+                # draw mask
+                rgb_mask = apply_color_mask(object_prediction.mask.bool_mask, color or (0, 0, 0))
+                image = cv2.addWeighted(image, 1, rgb_mask, 0.6, 0)
 
-            w, h = cv2.getTextSize(label, 0, fontScale=text_size, thickness=text_th)[0]  # label width, height
-            outside = p1[1] - h - 3 >= 0  # label fits outside box
-            p2 = p1[0] + w, p1[1] - h - 3 if outside else p1[1] + h + 3
-            # add bounding box text
-            cv2.rectangle(image, p1, p2, color, -1, cv2.LINE_AA)  # filled
-            cv2.putText(
+        # add bboxes to image if is_obb_pred=False
+        if not is_obb_pred:
+            bbox = object_prediction.bbox.to_xyxy()
+
+            # set bbox points
+            point1, point2 = (int(bbox[0]), int(bbox[1])), (int(bbox[2]), int(bbox[3]))
+            # visualize boxes
+            cv2.rectangle(
                 image,
-                label,
-                (p1[0], p1[1] - 2 if outside else p1[1] + h + 2),
-                0,
-                text_size,
-                (255, 255, 255),
-                thickness=text_th,
+                point1,
+                point2,
+                color=color or (0, 0, 0),
+                thickness=rect_th,
             )
 
+            if not hide_labels:
+                box_width, box_height = cv2.getTextSize(label, 0, fontScale=text_size, thickness=text_th)[
+                    0
+                ]  # label width, height
+                outside = point1[1] - box_height - 3 >= 0  # label fits outside box
+                point2 = point1[0] + box_width, point1[1] - box_height - 3 if outside else point1[1] + box_height + 3
+                # add bounding box text
+                cv2.rectangle(image, point1, point2, color or (0, 0, 0), -1, cv2.LINE_AA)  # filled
+                cv2.putText(
+                    image,
+                    label,
+                    (point1[0], point1[1] - 2 if outside else point1[1] + box_height + 2),
+                    0,
+                    text_size,
+                    (255, 255, 255),
+                    thickness=text_th,
+                )
+
     # export if output_dir is present
     if output_dir is not None:
         # export image with predictions
         Path(output_dir).mkdir(parents=True, exist_ok=True)
         # save inference result
-        save_path = str(Path(output_dir) / (file_name + "." + export_format))
+        save_path = str(Path(output_dir) / ((file_name or "") + "." + (export_format or "")))
         cv2.imwrite(save_path, cv2.cvtColor(image, cv2.COLOR_RGB2BGR))
 
     elapsed_time = time.time() - elapsed_time
     return {"image": image, "elapsed_time": elapsed_time}
 
 
-def get_coco_segmentation_from_bool_mask(bool_mask):
+def get_coco_segmentation_from_bool_mask(bool_mask: np.ndarray) -> List[List[float]]:
     """
     Convert boolean mask to coco segmentation format
     [
@@ -527,7 +642,7 @@ def get_coco_segmentation_from_bool_mask(bool_mask):
     # Generate polygons from mask
     mask = np.squeeze(bool_mask)
     mask = mask.astype(np.uint8)
-    mask = cv2.copyMakeBorder(mask, 1, 1, 1, 1, cv2.BORDER_CONSTANT, value=0)
+    mask = cv2.copyMakeBorder(mask, 1, 1, 1, 1, cv2.BORDER_CONSTANT, value=(0, 0, 0))
     polygons = cv2.findContours(mask, cv2.RETR_LIST, cv2.CHAIN_APPROX_SIMPLE, offset=(-1, -1))
     polygons = polygons[0] if len(polygons) == 2 else polygons[1]
     # Convert polygon to coco segmentation
@@ -540,21 +655,35 @@ def get_coco_segmentation_from_bool_mask(bool_mask):
     return coco_segmentation
 
 
-def get_bool_mask_from_coco_segmentation(coco_segmentation, width, height):
+def get_bool_mask_from_coco_segmentation(coco_segmentation: List[List[float]], width: int, height: int) -> np.ndarray:
     """
     Convert coco segmentation to 2D boolean mask of given height and width
+
+    Parameters:
+    - coco_segmentation: list of points representing the coco segmentation
+    - width: width of the boolean mask
+    - height: height of the boolean mask
+
+    Returns:
+    - bool_mask: 2D boolean mask of size (height, width)
     """
     size = [height, width]
     points = [np.array(point).reshape(-1, 2).round().astype(int) for point in coco_segmentation]
     bool_mask = np.zeros(size)
-    bool_mask = cv2.fillPoly(bool_mask, points, 1)
+    bool_mask = cv2.fillPoly(bool_mask, points, (1.0,))
     bool_mask.astype(bool)
     return bool_mask
 
 
-def get_bbox_from_bool_mask(bool_mask):
+def get_bbox_from_bool_mask(bool_mask: np.ndarray) -> Optional[List[int]]:
     """
-    Generate voc bbox ([xmin, ymin, xmax, ymax]) from given bool_mask (2D np.ndarray)
+    Generate VOC bounding box [xmin, ymin, xmax, ymax] from given boolean mask.
+
+    Args:
+        bool_mask (np.ndarray): 2D boolean mask.
+
+    Returns:
+        Optional[List[int]]: VOC bounding box [xmin, ymin, xmax, ymax] or None if no bounding box is found.
     """
     rows = np.any(bool_mask, axis=1)
     cols = np.any(bool_mask, axis=0)
@@ -573,6 +702,49 @@ def get_bbox_from_bool_mask(bool_mask):
     return [xmin, ymin, xmax, ymax]
 
 
+def get_bbox_from_coco_segmentation(coco_segmentation):
+    """
+    Generate voc box ([xmin, ymin, xmax, ymax]) from given coco segmentation
+    """
+    xs = []
+    ys = []
+    for segm in coco_segmentation:
+        xs.extend(segm[::2])
+        ys.extend(segm[1::2])
+    if len(xs) == 0 or len(ys) == 0:
+        return None
+    xmin = min(xs)
+    xmax = max(xs)
+    ymin = min(ys)
+    ymax = max(ys)
+    return [xmin, ymin, xmax, ymax]
+
+
+def get_coco_segmentation_from_obb_points(obb_points: np.ndarray) -> List[List[float]]:
+    """
+    Convert OBB (Oriented Bounding Box) points to COCO polygon format.
+
+    Args:
+        obb_points: np.ndarray
+            OBB points tensor from ultralytics.engine.results.OBB
+            Shape: (4, 2) containing 4 points with (x,y) coordinates each
+
+    Returns:
+        List[List[float]]: Polygon points in COCO format
+            [[x1, y1, x2, y2, x3, y3, x4, y4], [...], ...]
+    """
+    # Convert from (4,2) to [x1,y1,x2,y2,x3,y3,x4,y4] format
+    points = obb_points.reshape(-1).tolist()
+
+    # Create polygon from points and close it by repeating first point
+    polygons = []
+    # Add first point to end to close polygon
+    closed_polygon = points + [points[0], points[1]]
+    polygons.append(closed_polygon)
+
+    return polygons
+
+
 def normalize_numpy_image(image: np.ndarray):
     """
     Normalizes numpy image
@@ -591,31 +763,5 @@ def ipython_display(image: np.ndarray):
 
     image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
     _, ret = cv2.imencode(".png", image)
-    i = IPython.display.Image(data=ret)
-    IPython.display.display(i)
-
-
-def exif_transpose(image: Image.Image):
-    """
-    Transpose a PIL image accordingly if it has an EXIF Orientation tag.
-    Inplace version of https://github.com/python-pillow/Pillow/blob/master/src/PIL/ImageOps.py exif_transpose()
-    :param image: The image to transpose.
-    :return: An image.
-    """
-    exif = image.getexif()
-    orientation = exif.get(0x0112, 1)  # default 1
-    if orientation > 1:
-        method = {
-            2: Image.FLIP_LEFT_RIGHT,
-            3: Image.ROTATE_180,
-            4: Image.FLIP_TOP_BOTTOM,
-            5: Image.TRANSPOSE,
-            6: Image.ROTATE_270,
-            7: Image.TRANSVERSE,
-            8: Image.ROTATE_90,
-        }.get(orientation)
-        if method is not None:
-            image = image.transpose(method)
-            del exif[0x0112]
-            image.info["exif"] = exif.tobytes()
-    return image
+    i = IPython.display.Image(data=ret)  # type: ignore
+    IPython.display.display(i)  # type: ignore
